package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A CloudService.
 */
@Entity
@Table(name = "cloud_service")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class CloudService implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "acronym", nullable = false)
    private String acronym;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @NotNull
    @Column(name = "type", nullable = false)
    private String type;

    @Lob
    @Column(name = "description", nullable = false)
    private String description;

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "cloudService")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "targetValue", "resourceType", "cloudService", "securityControls" }, allowSetters = true)
    private Set<Resource> resources = new HashSet<>();

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "cloudServices" }, allowSetters = true)
    private CloudServiceProvider cloudServiceProvider;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "cloudServices" }, allowSetters = true)
    private SecurityControlFramework securityControlFramework;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public CloudService id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getAcronym() {
        return this.acronym;
    }

    public CloudService acronym(String acronym) {
        this.setAcronym(acronym);
        return this;
    }

    public void setAcronym(String acronym) {
        this.acronym = acronym;
    }

    public String getName() {
        return this.name;
    }

    public CloudService name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getType() {
        return this.type;
    }

    public CloudService type(String type) {
        this.setType(type);
        return this;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getDescription() {
        return this.description;
    }

    public CloudService description(String description) {
        this.setDescription(description);
        return this;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Set<Resource> getResources() {
        return this.resources;
    }

    public void setResources(Set<Resource> resources) {
        if (this.resources != null) {
            this.resources.forEach(i -> i.setCloudService(null));
        }
        if (resources != null) {
            resources.forEach(i -> i.setCloudService(this));
        }
        this.resources = resources;
    }

    public CloudService resources(Set<Resource> resources) {
        this.setResources(resources);
        return this;
    }

    public CloudService addResource(Resource resource) {
        this.resources.add(resource);
        resource.setCloudService(this);
        return this;
    }

    public CloudService removeResource(Resource resource) {
        this.resources.remove(resource);
        resource.setCloudService(null);
        return this;
    }

    public CloudServiceProvider getCloudServiceProvider() {
        return this.cloudServiceProvider;
    }

    public void setCloudServiceProvider(CloudServiceProvider cloudServiceProvider) {
        this.cloudServiceProvider = cloudServiceProvider;
    }

    public CloudService cloudServiceProvider(CloudServiceProvider cloudServiceProvider) {
        this.setCloudServiceProvider(cloudServiceProvider);
        return this;
    }

    public SecurityControlFramework getSecurityControlFramework() {
        return this.securityControlFramework;
    }

    public void setSecurityControlFramework(SecurityControlFramework securityControlFramework) {
        this.securityControlFramework = securityControlFramework;
    }

    public CloudService securityControlFramework(SecurityControlFramework securityControlFramework) {
        this.setSecurityControlFramework(securityControlFramework);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CloudService)) {
            return false;
        }
        return getId() != null && getId().equals(((CloudService) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "CloudService{" +
            "id=" + getId() +
            ", acronym='" + getAcronym() + "'" +
            ", name='" + getName() + "'" +
            ", type='" + getType() + "'" +
            ", description='" + getDescription() + "'" +
            "}";
    }
}
