package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A CloudServiceProvider.
 */
@Entity
@Table(name = "cloud_service_provider")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class CloudServiceProvider implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "acronym", nullable = false)
    private String acronym;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @Lob
    @Column(name = "contact_details", nullable = false)
    private String contactDetails;

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "cloudServiceProvider")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "resources", "cloudServiceProvider", "securityControlFramework" }, allowSetters = true)
    private Set<CloudService> cloudServices = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public CloudServiceProvider id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getAcronym() {
        return this.acronym;
    }

    public CloudServiceProvider acronym(String acronym) {
        this.setAcronym(acronym);
        return this;
    }

    public void setAcronym(String acronym) {
        this.acronym = acronym;
    }

    public String getName() {
        return this.name;
    }

    public CloudServiceProvider name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getContactDetails() {
        return this.contactDetails;
    }

    public CloudServiceProvider contactDetails(String contactDetails) {
        this.setContactDetails(contactDetails);
        return this;
    }

    public void setContactDetails(String contactDetails) {
        this.contactDetails = contactDetails;
    }

    public Set<CloudService> getCloudServices() {
        return this.cloudServices;
    }

    public void setCloudServices(Set<CloudService> cloudServices) {
        if (this.cloudServices != null) {
            this.cloudServices.forEach(i -> i.setCloudServiceProvider(null));
        }
        if (cloudServices != null) {
            cloudServices.forEach(i -> i.setCloudServiceProvider(this));
        }
        this.cloudServices = cloudServices;
    }

    public CloudServiceProvider cloudServices(Set<CloudService> cloudServices) {
        this.setCloudServices(cloudServices);
        return this;
    }

    public CloudServiceProvider addCloudService(CloudService cloudService) {
        this.cloudServices.add(cloudService);
        cloudService.setCloudServiceProvider(this);
        return this;
    }

    public CloudServiceProvider removeCloudService(CloudService cloudService) {
        this.cloudServices.remove(cloudService);
        cloudService.setCloudServiceProvider(null);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CloudServiceProvider)) {
            return false;
        }
        return getId() != null && getId().equals(((CloudServiceProvider) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "CloudServiceProvider{" +
            "id=" + getId() +
            ", acronym='" + getAcronym() + "'" +
            ", name='" + getName() + "'" +
            ", contactDetails='" + getContactDetails() + "'" +
            "}";
    }
}
