package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A Question.
 */
@Entity
@Table(name = "question")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class Question implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "code", nullable = false)
    private String code;

    @NotNull
    @Column(name = "question", nullable = false)
    private String question;

    @NotNull
    @Column(name = "default_evidence", nullable = false)
    private String defaultEvidence;

    @NotNull
    @Column(name = "default_comment", nullable = false)
    private String defaultComment;

    @ManyToOne(fetch = FetchType.LAZY)
    private QuestionAssuranceLevel questionAssuranceLevel;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "similarControls", "securityControlCategory", "resources" }, allowSetters = true)
    private SecurityControl securityControl;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "securityControl", "securityMetrics" }, allowSetters = true)
    private Tom tom;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "cloudServices" }, allowSetters = true)
    private SecurityControlFramework securityControlFramework;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Question id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCode() {
        return this.code;
    }

    public Question code(String code) {
        this.setCode(code);
        return this;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getQuestion() {
        return this.question;
    }

    public Question question(String question) {
        this.setQuestion(question);
        return this;
    }

    public void setQuestion(String question) {
        this.question = question;
    }

    public String getDefaultEvidence() {
        return this.defaultEvidence;
    }

    public Question defaultEvidence(String defaultEvidence) {
        this.setDefaultEvidence(defaultEvidence);
        return this;
    }

    public void setDefaultEvidence(String defaultEvidence) {
        this.defaultEvidence = defaultEvidence;
    }

    public String getDefaultComment() {
        return this.defaultComment;
    }

    public Question defaultComment(String defaultComment) {
        this.setDefaultComment(defaultComment);
        return this;
    }

    public void setDefaultComment(String defaultComment) {
        this.defaultComment = defaultComment;
    }

    public QuestionAssuranceLevel getQuestionAssuranceLevel() {
        return this.questionAssuranceLevel;
    }

    public void setQuestionAssuranceLevel(QuestionAssuranceLevel questionAssuranceLevel) {
        this.questionAssuranceLevel = questionAssuranceLevel;
    }

    public Question questionAssuranceLevel(QuestionAssuranceLevel questionAssuranceLevel) {
        this.setQuestionAssuranceLevel(questionAssuranceLevel);
        return this;
    }

    public SecurityControl getSecurityControl() {
        return this.securityControl;
    }

    public void setSecurityControl(SecurityControl securityControl) {
        this.securityControl = securityControl;
    }

    public Question securityControl(SecurityControl securityControl) {
        this.setSecurityControl(securityControl);
        return this;
    }

    public Tom getTom() {
        return this.tom;
    }

    public void setTom(Tom tom) {
        this.tom = tom;
    }

    public Question tom(Tom tom) {
        this.setTom(tom);
        return this;
    }

    public SecurityControlFramework getSecurityControlFramework() {
        return this.securityControlFramework;
    }

    public void setSecurityControlFramework(SecurityControlFramework securityControlFramework) {
        this.securityControlFramework = securityControlFramework;
    }

    public Question securityControlFramework(SecurityControlFramework securityControlFramework) {
        this.setSecurityControlFramework(securityControlFramework);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Question)) {
            return false;
        }
        return getId() != null && getId().equals(((Question) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Question{" +
            "id=" + getId() +
            ", code='" + getCode() + "'" +
            ", question='" + getQuestion() + "'" +
            ", defaultEvidence='" + getDefaultEvidence() + "'" +
            ", defaultComment='" + getDefaultComment() + "'" +
            "}";
    }
}
