package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.time.Instant;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A Questionnaire.
 */
@Entity
@Table(name = "questionnaire")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class Questionnaire implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @Lob
    @Column(name = "evidences", nullable = false)
    private String evidences;

    @Lob
    @Column(name = "comments", nullable = false)
    private String comments;

    @NotNull
    @Column(name = "last_update", nullable = false)
    private Instant lastUpdate;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "cloudServices" }, allowSetters = true)
    private SecurityControlFramework securityControlFramework;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "similarControls", "securityControlCategory", "resources" }, allowSetters = true)
    private SecurityControl securityControl;

    @ManyToOne(fetch = FetchType.LAZY)
    private QuestionAssuranceLevel questionAssuranceLevel;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "questionAssuranceLevel", "securityControl", "tom", "securityControlFramework" }, allowSetters = true)
    private Question question;

    @ManyToOne(optional = false)
    @NotNull
    private QuestionAnswer questionAnswer;

    @ManyToOne(optional = false)
    @NotNull
    private User user;

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Questionnaire id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return this.name;
    }

    public Questionnaire name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEvidences() {
        return this.evidences;
    }

    public Questionnaire evidences(String evidences) {
        this.setEvidences(evidences);
        return this;
    }

    public void setEvidences(String evidences) {
        this.evidences = evidences;
    }

    public String getComments() {
        return this.comments;
    }

    public Questionnaire comments(String comments) {
        this.setComments(comments);
        return this;
    }

    public void setComments(String comments) {
        this.comments = comments;
    }

    public Instant getLastUpdate() {
        return this.lastUpdate;
    }

    public Questionnaire lastUpdate(Instant lastUpdate) {
        this.setLastUpdate(lastUpdate);
        return this;
    }

    public void setLastUpdate(Instant lastUpdate) {
        this.lastUpdate = lastUpdate;
    }

    public SecurityControlFramework getSecurityControlFramework() {
        return this.securityControlFramework;
    }

    public void setSecurityControlFramework(SecurityControlFramework securityControlFramework) {
        this.securityControlFramework = securityControlFramework;
    }

    public Questionnaire securityControlFramework(SecurityControlFramework securityControlFramework) {
        this.setSecurityControlFramework(securityControlFramework);
        return this;
    }

    public SecurityControl getSecurityControl() {
        return this.securityControl;
    }

    public void setSecurityControl(SecurityControl securityControl) {
        this.securityControl = securityControl;
    }

    public Questionnaire securityControl(SecurityControl securityControl) {
        this.setSecurityControl(securityControl);
        return this;
    }

    public QuestionAssuranceLevel getQuestionAssuranceLevel() {
        return this.questionAssuranceLevel;
    }

    public void setQuestionAssuranceLevel(QuestionAssuranceLevel questionAssuranceLevel) {
        this.questionAssuranceLevel = questionAssuranceLevel;
    }

    public Questionnaire questionAssuranceLevel(QuestionAssuranceLevel questionAssuranceLevel) {
        this.setQuestionAssuranceLevel(questionAssuranceLevel);
        return this;
    }

    public Question getQuestion() {
        return this.question;
    }

    public void setQuestion(Question question) {
        this.question = question;
    }

    public Questionnaire question(Question question) {
        this.setQuestion(question);
        return this;
    }

    public QuestionAnswer getQuestionAnswer() {
        return this.questionAnswer;
    }

    public void setQuestionAnswer(QuestionAnswer questionAnswer) {
        this.questionAnswer = questionAnswer;
    }

    public Questionnaire questionAnswer(QuestionAnswer questionAnswer) {
        this.setQuestionAnswer(questionAnswer);
        return this;
    }

    public User getUser() {
        return this.user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public Questionnaire user(User user) {
        this.setUser(user);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Questionnaire)) {
            return false;
        }
        return getId() != null && getId().equals(((Questionnaire) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Questionnaire{" +
            "id=" + getId() +
            ", name='" + getName() + "'" +
            ", evidences='" + getEvidences() + "'" +
            ", comments='" + getComments() + "'" +
            ", lastUpdate='" + getLastUpdate() + "'" +
            "}";
    }
}
