package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A Resource.
 */
@Entity
@Table(name = "resource")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class Resource implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "acronym", nullable = false)
    private String acronym;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @Column(name = "confidentiality_risk")
    private String confidentialityRisk;

    @Column(name = "availability_risk")
    private String availabilityRisk;

    @Column(name = "integrity_risk")
    private String integrityRisk;

    @ManyToOne(optional = false)
    @NotNull
    private TargetValue targetValue;

    @ManyToOne(optional = false)
    @NotNull
    private ResourceType resourceType;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "resources", "cloudServiceProvider", "securityControlFramework" }, allowSetters = true)
    private CloudService cloudService;

    @ManyToMany(fetch = FetchType.LAZY, mappedBy = "resources")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "similarControls", "securityControlCategory", "resources" }, allowSetters = true)
    private Set<SecurityControl> securityControls = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Resource id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getAcronym() {
        return this.acronym;
    }

    public Resource acronym(String acronym) {
        this.setAcronym(acronym);
        return this;
    }

    public void setAcronym(String acronym) {
        this.acronym = acronym;
    }

    public String getName() {
        return this.name;
    }

    public Resource name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getConfidentialityRisk() {
        return this.confidentialityRisk;
    }

    public Resource confidentialityRisk(String confidentialityRisk) {
        this.setConfidentialityRisk(confidentialityRisk);
        return this;
    }

    public void setConfidentialityRisk(String confidentialityRisk) {
        this.confidentialityRisk = confidentialityRisk;
    }

    public String getAvailabilityRisk() {
        return this.availabilityRisk;
    }

    public Resource availabilityRisk(String availabilityRisk) {
        this.setAvailabilityRisk(availabilityRisk);
        return this;
    }

    public void setAvailabilityRisk(String availabilityRisk) {
        this.availabilityRisk = availabilityRisk;
    }

    public String getIntegrityRisk() {
        return this.integrityRisk;
    }

    public Resource integrityRisk(String integrityRisk) {
        this.setIntegrityRisk(integrityRisk);
        return this;
    }

    public void setIntegrityRisk(String integrityRisk) {
        this.integrityRisk = integrityRisk;
    }

    public TargetValue getTargetValue() {
        return this.targetValue;
    }

    public void setTargetValue(TargetValue targetValue) {
        this.targetValue = targetValue;
    }

    public Resource targetValue(TargetValue targetValue) {
        this.setTargetValue(targetValue);
        return this;
    }

    public ResourceType getResourceType() {
        return this.resourceType;
    }

    public void setResourceType(ResourceType resourceType) {
        this.resourceType = resourceType;
    }

    public Resource resourceType(ResourceType resourceType) {
        this.setResourceType(resourceType);
        return this;
    }

    public CloudService getCloudService() {
        return this.cloudService;
    }

    public void setCloudService(CloudService cloudService) {
        this.cloudService = cloudService;
    }

    public Resource cloudService(CloudService cloudService) {
        this.setCloudService(cloudService);
        return this;
    }

    public Set<SecurityControl> getSecurityControls() {
        return this.securityControls;
    }

    public void setSecurityControls(Set<SecurityControl> securityControls) {
        if (this.securityControls != null) {
            this.securityControls.forEach(i -> i.removeResource(this));
        }
        if (securityControls != null) {
            securityControls.forEach(i -> i.addResource(this));
        }
        this.securityControls = securityControls;
    }

    public Resource securityControls(Set<SecurityControl> securityControls) {
        this.setSecurityControls(securityControls);
        return this;
    }

    public Resource addSecurityControl(SecurityControl securityControl) {
        this.securityControls.add(securityControl);
        securityControl.getResources().add(this);
        return this;
    }

    public Resource removeSecurityControl(SecurityControl securityControl) {
        this.securityControls.remove(securityControl);
        securityControl.getResources().remove(this);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Resource)) {
            return false;
        }
        return getId() != null && getId().equals(((Resource) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Resource{" +
            "id=" + getId() +
            ", acronym='" + getAcronym() + "'" +
            ", name='" + getName() + "'" +
            ", confidentialityRisk='" + getConfidentialityRisk() + "'" +
            ", availabilityRisk='" + getAvailabilityRisk() + "'" +
            ", integrityRisk='" + getIntegrityRisk() + "'" +
            "}";
    }
}
