package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A SecurityControl.
 */
@Entity
@Table(name = "security_control")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class SecurityControl implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "code", nullable = false)
    private String code;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @NotNull
    @Column(name = "objective", nullable = false)
    private String objective;

    @Lob
    @Column(name = "description", nullable = false)
    private String description;

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "securityControl")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "securityControl" }, allowSetters = true)
    private Set<SimilarControl> similarControls = new HashSet<>();

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "securityControlFramework" }, allowSetters = true)
    private SecurityControlCategory securityControlCategory;

    @ManyToMany(fetch = FetchType.LAZY)
    @JoinTable(
        name = "rel_security_control__resource",
        joinColumns = @JoinColumn(name = "security_control_id"),
        inverseJoinColumns = @JoinColumn(name = "resource_id")
    )
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "targetValue", "resourceType", "cloudService", "securityControls" }, allowSetters = true)
    private Set<Resource> resources = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public SecurityControl id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCode() {
        return this.code;
    }

    public SecurityControl code(String code) {
        this.setCode(code);
        return this;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getName() {
        return this.name;
    }

    public SecurityControl name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getObjective() {
        return this.objective;
    }

    public SecurityControl objective(String objective) {
        this.setObjective(objective);
        return this;
    }

    public void setObjective(String objective) {
        this.objective = objective;
    }

    public String getDescription() {
        return this.description;
    }

    public SecurityControl description(String description) {
        this.setDescription(description);
        return this;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Set<SimilarControl> getSimilarControls() {
        return this.similarControls;
    }

    public void setSimilarControls(Set<SimilarControl> similarControls) {
        if (this.similarControls != null) {
            this.similarControls.forEach(i -> i.setSecurityControl(null));
        }
        if (similarControls != null) {
            similarControls.forEach(i -> i.setSecurityControl(this));
        }
        this.similarControls = similarControls;
    }

    public SecurityControl similarControls(Set<SimilarControl> similarControls) {
        this.setSimilarControls(similarControls);
        return this;
    }

    public SecurityControl addSimilarControls(SimilarControl similarControl) {
        this.similarControls.add(similarControl);
        similarControl.setSecurityControl(this);
        return this;
    }

    public SecurityControl removeSimilarControls(SimilarControl similarControl) {
        this.similarControls.remove(similarControl);
        similarControl.setSecurityControl(null);
        return this;
    }

    public SecurityControlCategory getSecurityControlCategory() {
        return this.securityControlCategory;
    }

    public void setSecurityControlCategory(SecurityControlCategory securityControlCategory) {
        this.securityControlCategory = securityControlCategory;
    }

    public SecurityControl securityControlCategory(SecurityControlCategory securityControlCategory) {
        this.setSecurityControlCategory(securityControlCategory);
        return this;
    }

    public Set<Resource> getResources() {
        return this.resources;
    }

    public void setResources(Set<Resource> resources) {
        this.resources = resources;
    }

    public SecurityControl resources(Set<Resource> resources) {
        this.setResources(resources);
        return this;
    }

    public SecurityControl addResource(Resource resource) {
        this.resources.add(resource);
        return this;
    }

    public SecurityControl removeResource(Resource resource) {
        this.resources.remove(resource);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SecurityControl)) {
            return false;
        }
        return getId() != null && getId().equals(((SecurityControl) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "SecurityControl{" +
            "id=" + getId() +
            ", code='" + getCode() + "'" +
            ", name='" + getName() + "'" +
            ", objective='" + getObjective() + "'" +
            ", description='" + getDescription() + "'" +
            "}";
    }
}
