package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A SecurityControlFramework.
 */
@Entity
@Table(name = "security_control_framework")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class SecurityControlFramework implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @Lob
    @Column(name = "description", nullable = false)
    private String description;

    @NotNull
    @Column(name = "version", nullable = false)
    private String version;

    @OneToMany(fetch = FetchType.LAZY, mappedBy = "securityControlFramework")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "resources", "cloudServiceProvider", "securityControlFramework" }, allowSetters = true)
    private Set<CloudService> cloudServices = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public SecurityControlFramework id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return this.name;
    }

    public SecurityControlFramework name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return this.description;
    }

    public SecurityControlFramework description(String description) {
        this.setDescription(description);
        return this;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getVersion() {
        return this.version;
    }

    public SecurityControlFramework version(String version) {
        this.setVersion(version);
        return this;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public Set<CloudService> getCloudServices() {
        return this.cloudServices;
    }

    public void setCloudServices(Set<CloudService> cloudServices) {
        if (this.cloudServices != null) {
            this.cloudServices.forEach(i -> i.setSecurityControlFramework(null));
        }
        if (cloudServices != null) {
            cloudServices.forEach(i -> i.setSecurityControlFramework(this));
        }
        this.cloudServices = cloudServices;
    }

    public SecurityControlFramework cloudServices(Set<CloudService> cloudServices) {
        this.setCloudServices(cloudServices);
        return this;
    }

    public SecurityControlFramework addCloudService(CloudService cloudService) {
        this.cloudServices.add(cloudService);
        cloudService.setSecurityControlFramework(this);
        return this;
    }

    public SecurityControlFramework removeCloudService(CloudService cloudService) {
        this.cloudServices.remove(cloudService);
        cloudService.setSecurityControlFramework(null);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SecurityControlFramework)) {
            return false;
        }
        return getId() != null && getId().equals(((SecurityControlFramework) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "SecurityControlFramework{" +
            "id=" + getId() +
            ", name='" + getName() + "'" +
            ", description='" + getDescription() + "'" +
            ", version='" + getVersion() + "'" +
            "}";
    }
}
