package com.emerald.rcm.backend.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A SecurityMetric.
 */
@Entity
@Table(name = "security_metric")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class SecurityMetric implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @Column(name = "metric_id")
    private String metricId;

    @NotNull
    @Column(name = "category", nullable = false)
    private String category;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @NotNull
    @Column(name = "source", nullable = false)
    private String source;

    @Lob
    @Column(name = "description", nullable = false)
    private String description;

    @NotNull
    @Column(name = "scale", nullable = false)
    private String scale;

    @NotNull
    @Column(name = "operator", nullable = false)
    private String operator;

    @NotNull
    @Column(name = "target_value", nullable = false)
    private String targetValue;

    @NotNull
    @Column(name = "target_value_datatype", nullable = false)
    private String targetValueDatatype;

    @NotNull
    @Column(name = "jhi_interval", nullable = false)
    private String interval;

    @Column(name = "target_resource_type")
    private String targetResourceType;

    @Column(name = "security_feature")
    private String securityFeature;

    @Column(name = "keywords")
    private String keywords;

    @ManyToOne(fetch = FetchType.LAZY)
    @JsonIgnoreProperties(value = { "tom" }, allowSetters = true)
    private ReferenceTom referenceTom;

    @ManyToOne(fetch = FetchType.LAZY)
    private ResourceType resourceType;

    @ManyToMany(fetch = FetchType.LAZY, mappedBy = "securityMetrics")
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "securityControl", "securityMetrics" }, allowSetters = true)
    private Set<Tom> toms = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public SecurityMetric id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getMetricId() {
        return this.metricId;
    }

    public SecurityMetric metricId(String metricId) {
        this.setMetricId(metricId);
        return this;
    }

    public void setMetricId(String metricId) {
        this.metricId = metricId;
    }

    public String getCategory() {
        return this.category;
    }

    public SecurityMetric category(String category) {
        this.setCategory(category);
        return this;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public String getName() {
        return this.name;
    }

    public SecurityMetric name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getSource() {
        return this.source;
    }

    public SecurityMetric source(String source) {
        this.setSource(source);
        return this;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getDescription() {
        return this.description;
    }

    public SecurityMetric description(String description) {
        this.setDescription(description);
        return this;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getScale() {
        return this.scale;
    }

    public SecurityMetric scale(String scale) {
        this.setScale(scale);
        return this;
    }

    public void setScale(String scale) {
        this.scale = scale;
    }

    public String getOperator() {
        return this.operator;
    }

    public SecurityMetric operator(String operator) {
        this.setOperator(operator);
        return this;
    }

    public void setOperator(String operator) {
        this.operator = operator;
    }

    public String getTargetValue() {
        return this.targetValue;
    }

    public SecurityMetric targetValue(String targetValue) {
        this.setTargetValue(targetValue);
        return this;
    }

    public void setTargetValue(String targetValue) {
        this.targetValue = targetValue;
    }

    public String getTargetValueDatatype() {
        return this.targetValueDatatype;
    }

    public SecurityMetric targetValueDatatype(String targetValueDatatype) {
        this.setTargetValueDatatype(targetValueDatatype);
        return this;
    }

    public void setTargetValueDatatype(String targetValueDatatype) {
        this.targetValueDatatype = targetValueDatatype;
    }

    public String getInterval() {
        return this.interval;
    }

    public SecurityMetric interval(String interval) {
        this.setInterval(interval);
        return this;
    }

    public void setInterval(String interval) {
        this.interval = interval;
    }

    public String getTargetResourceType() {
        return this.targetResourceType;
    }

    public SecurityMetric targetResourceType(String targetResourceType) {
        this.setTargetResourceType(targetResourceType);
        return this;
    }

    public void setTargetResourceType(String targetResourceType) {
        this.targetResourceType = targetResourceType;
    }

    public String getSecurityFeature() {
        return this.securityFeature;
    }

    public SecurityMetric securityFeature(String securityFeature) {
        this.setSecurityFeature(securityFeature);
        return this;
    }

    public void setSecurityFeature(String securityFeature) {
        this.securityFeature = securityFeature;
    }

    public String getKeywords() {
        return this.keywords;
    }

    public SecurityMetric keywords(String keywords) {
        this.setKeywords(keywords);
        return this;
    }

    public void setKeywords(String keywords) {
        this.keywords = keywords;
    }

    public ReferenceTom getReferenceTom() {
        return this.referenceTom;
    }

    public void setReferenceTom(ReferenceTom referenceTom) {
        this.referenceTom = referenceTom;
    }

    public SecurityMetric referenceTom(ReferenceTom referenceTom) {
        this.setReferenceTom(referenceTom);
        return this;
    }

    public ResourceType getResourceType() {
        return this.resourceType;
    }

    public void setResourceType(ResourceType resourceType) {
        this.resourceType = resourceType;
    }

    public SecurityMetric resourceType(ResourceType resourceType) {
        this.setResourceType(resourceType);
        return this;
    }

    public Set<Tom> getToms() {
        return this.toms;
    }

    public void setToms(Set<Tom> toms) {
        if (this.toms != null) {
            this.toms.forEach(i -> i.removeSecurityMetric(this));
        }
        if (toms != null) {
            toms.forEach(i -> i.addSecurityMetric(this));
        }
        this.toms = toms;
    }

    public SecurityMetric toms(Set<Tom> toms) {
        this.setToms(toms);
        return this;
    }

    public SecurityMetric addTom(Tom tom) {
        this.toms.add(tom);
        tom.getSecurityMetrics().add(this);
        return this;
    }

    public SecurityMetric removeTom(Tom tom) {
        this.toms.remove(tom);
        tom.getSecurityMetrics().remove(this);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SecurityMetric)) {
            return false;
        }
        return getId() != null && getId().equals(((SecurityMetric) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "SecurityMetric{" +
            "id=" + getId() +
            ", metricId='" + getMetricId() + "'" +
            ", category='" + getCategory() + "'" +
            ", name='" + getName() + "'" +
            ", source='" + getSource() + "'" +
            ", description='" + getDescription() + "'" +
            ", scale='" + getScale() + "'" +
            ", operator='" + getOperator() + "'" +
            ", targetValue='" + getTargetValue() + "'" +
            ", targetValueDatatype='" + getTargetValueDatatype() + "'" +
            ", interval='" + getInterval() + "'" +
            ", targetResourceType='" + getTargetResourceType() + "'" +
            ", securityFeature='" + getSecurityFeature() + "'" +
            ", keywords='" + getKeywords() + "'" +
            "}";
    }
}
