package com.emerald.rcm.backend.domain;

import com.emerald.rcm.backend.domain.enumeration.AssuranceLevel;
import com.emerald.rcm.backend.domain.enumeration.Type;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import jakarta.persistence.*;
import jakarta.validation.constraints.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A Tom.
 */
@Entity
@Table(name = "tom")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("common-java:DuplicatedBlocks")
public class Tom implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sequenceGenerator")
    @SequenceGenerator(name = "sequenceGenerator")
    @Column(name = "id")
    private Long id;

    @NotNull
    @Column(name = "code", nullable = false)
    private String code;

    @NotNull
    @Column(name = "name", nullable = false)
    private String name;

    @Lob
    @Column(name = "description", nullable = false)
    private String description;

    @Enumerated(EnumType.STRING)
    @Column(name = "assurance_level")
    private AssuranceLevel assuranceLevel;

    @NotNull
    @Enumerated(EnumType.STRING)
    @Column(name = "type", nullable = false)
    private Type type;

    @Lob
    @Column(name = "guidance", nullable = false)
    private String guidance;

    @ManyToOne(optional = false)
    @NotNull
    @JsonIgnoreProperties(value = { "similarControls", "securityControlCategory", "resources" }, allowSetters = true)
    private SecurityControl securityControl;

    @ManyToMany(fetch = FetchType.LAZY)
    @JoinTable(
        name = "rel_tom__security_metric",
        joinColumns = @JoinColumn(name = "tom_id"),
        inverseJoinColumns = @JoinColumn(name = "security_metric_id")
    )
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "referenceTom", "resourceType", "toms" }, allowSetters = true)
    private Set<SecurityMetric> securityMetrics = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Tom id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCode() {
        return this.code;
    }

    public Tom code(String code) {
        this.setCode(code);
        return this;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getName() {
        return this.name;
    }

    public Tom name(String name) {
        this.setName(name);
        return this;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return this.description;
    }

    public Tom description(String description) {
        this.setDescription(description);
        return this;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public AssuranceLevel getAssuranceLevel() {
        return this.assuranceLevel;
    }

    public Tom assuranceLevel(AssuranceLevel assuranceLevel) {
        this.setAssuranceLevel(assuranceLevel);
        return this;
    }

    public void setAssuranceLevel(AssuranceLevel assuranceLevel) {
        this.assuranceLevel = assuranceLevel;
    }

    public Type getType() {
        return this.type;
    }

    public Tom type(Type type) {
        this.setType(type);
        return this;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public String getGuidance() {
        return this.guidance;
    }

    public Tom guidance(String guidance) {
        this.setGuidance(guidance);
        return this;
    }

    public void setGuidance(String guidance) {
        this.guidance = guidance;
    }

    public SecurityControl getSecurityControl() {
        return this.securityControl;
    }

    public void setSecurityControl(SecurityControl securityControl) {
        this.securityControl = securityControl;
    }

    public Tom securityControl(SecurityControl securityControl) {
        this.setSecurityControl(securityControl);
        return this;
    }

    public Set<SecurityMetric> getSecurityMetrics() {
        return this.securityMetrics;
    }

    public void setSecurityMetrics(Set<SecurityMetric> securityMetrics) {
        this.securityMetrics = securityMetrics;
    }

    public Tom securityMetrics(Set<SecurityMetric> securityMetrics) {
        this.setSecurityMetrics(securityMetrics);
        return this;
    }

    public Tom addSecurityMetric(SecurityMetric securityMetric) {
        this.securityMetrics.add(securityMetric);
        return this;
    }

    public Tom removeSecurityMetric(SecurityMetric securityMetric) {
        this.securityMetrics.remove(securityMetric);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Tom)) {
            return false;
        }
        return getId() != null && getId().equals(((Tom) o).getId());
    }

    @Override
    public int hashCode() {
        // see https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Tom{" +
            "id=" + getId() +
            ", code='" + getCode() + "'" +
            ", name='" + getName() + "'" +
            ", description='" + getDescription() + "'" +
            ", assuranceLevel='" + getAssuranceLevel() + "'" +
            ", type='" + getType() + "'" +
            ", guidance='" + getGuidance() + "'" +
            "}";
    }
}
