package com.emerald.rcm.backend.repository;

import com.emerald.rcm.backend.domain.CloudService;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.*;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

/**
 * Spring Data JPA repository for the CloudService entity.
 */
@Repository
public interface CloudServiceRepository extends JpaRepository<CloudService, Long> {
    default Optional<CloudService> findOneWithEagerRelationships(Long id) {
        return this.findOneWithToOneRelationships(id);
    }

    default List<CloudService> findAllWithEagerRelationships() {
        return this.findAllWithToOneRelationships();
    }

    default Page<CloudService> findAllWithEagerRelationships(Pageable pageable) {
        return this.findAllWithToOneRelationships(pageable);
    }

    @Query(
        value = "select cloudService from CloudService cloudService left join fetch cloudService.cloudServiceProvider left join fetch cloudService.securityControlFramework",
        countQuery = "select count(cloudService) from CloudService cloudService"
    )
    Page<CloudService> findAllWithToOneRelationships(Pageable pageable);

    @Query(
        "select cloudService from CloudService cloudService left join fetch cloudService.cloudServiceProvider left join fetch cloudService.securityControlFramework"
    )
    List<CloudService> findAllWithToOneRelationships();

    @Query(
        "select cloudService from CloudService cloudService left join fetch cloudService.cloudServiceProvider left join fetch cloudService.securityControlFramework where cloudService.id =:id"
    )
    Optional<CloudService> findOneWithToOneRelationships(@Param("id") Long id);
}
