package com.emerald.rcm.backend.repository;

import com.emerald.rcm.backend.domain.Questionnaire;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.*;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

/**
 * Spring Data JPA repository for the Questionnaire entity.
 */
@Repository
public interface QuestionnaireRepository extends JpaRepository<Questionnaire, Long> {
    @Query("select questionnaire from Questionnaire questionnaire where questionnaire.user.login = ?#{authentication.name}")
    List<Questionnaire> findByUserIsCurrentUser();

    default Optional<Questionnaire> findOneWithEagerRelationships(Long id) {
        return this.findOneWithToOneRelationships(id);
    }

    default List<Questionnaire> findAllWithEagerRelationships() {
        return this.findAllWithToOneRelationships();
    }

    default Page<Questionnaire> findAllWithEagerRelationships(Pageable pageable) {
        return this.findAllWithToOneRelationships(pageable);
    }

    @Query(
        value = "select questionnaire from Questionnaire questionnaire left join fetch questionnaire.securityControlFramework left join fetch questionnaire.securityControl left join fetch questionnaire.questionAssuranceLevel left join fetch questionnaire.question left join fetch questionnaire.questionAnswer left join fetch questionnaire.user",
        countQuery = "select count(questionnaire) from Questionnaire questionnaire"
    )
    Page<Questionnaire> findAllWithToOneRelationships(Pageable pageable);

    @Query(
        "select questionnaire from Questionnaire questionnaire left join fetch questionnaire.securityControlFramework left join fetch questionnaire.securityControl left join fetch questionnaire.questionAssuranceLevel left join fetch questionnaire.question left join fetch questionnaire.questionAnswer left join fetch questionnaire.user"
    )
    List<Questionnaire> findAllWithToOneRelationships();

    @Query(
        "select questionnaire from Questionnaire questionnaire left join fetch questionnaire.securityControlFramework left join fetch questionnaire.securityControl left join fetch questionnaire.questionAssuranceLevel left join fetch questionnaire.question left join fetch questionnaire.questionAnswer left join fetch questionnaire.user where questionnaire.id =:id"
    )
    Optional<Questionnaire> findOneWithToOneRelationships(@Param("id") Long id);
}
