package com.emerald.rcm.backend.repository;

import com.emerald.rcm.backend.domain.SecurityControl;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.*;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

/**
 * Spring Data JPA repository for the SecurityControl entity.
 *
 * When extending this class, extend SecurityControlRepositoryWithBagRelationships too.
 * For more information refer to https://github.com/jhipster/generator-jhipster/issues/17990.
 */
@Repository
public interface SecurityControlRepository extends SecurityControlRepositoryWithBagRelationships, JpaRepository<SecurityControl, Long> {
    default Optional<SecurityControl> findOneWithEagerRelationships(Long id) {
        return this.fetchBagRelationships(this.findOneWithToOneRelationships(id));
    }

    default List<SecurityControl> findAllWithEagerRelationships() {
        return this.fetchBagRelationships(this.findAllWithToOneRelationships());
    }

    default Page<SecurityControl> findAllWithEagerRelationships(Pageable pageable) {
        return this.fetchBagRelationships(this.findAllWithToOneRelationships(pageable));
    }

    @Query(
        value = "select securityControl from SecurityControl securityControl left join fetch securityControl.securityControlCategory",
        countQuery = "select count(securityControl) from SecurityControl securityControl"
    )
    Page<SecurityControl> findAllWithToOneRelationships(Pageable pageable);

    @Query("select securityControl from SecurityControl securityControl left join fetch securityControl.securityControlCategory")
    List<SecurityControl> findAllWithToOneRelationships();

    @Query(
        "select securityControl from SecurityControl securityControl left join fetch securityControl.securityControlCategory where securityControl.id =:id"
    )
    Optional<SecurityControl> findOneWithToOneRelationships(@Param("id") Long id);
}
