package com.emerald.rcm.backend.repository;

import com.emerald.rcm.backend.domain.Tom;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.stream.IntStream;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;

/**
 * Utility repository to load bag relationships based on https://vladmihalcea.com/hibernate-multiplebagfetchexception/
 */
public class TomRepositoryWithBagRelationshipsImpl implements TomRepositoryWithBagRelationships {

    private static final String ID_PARAMETER = "id";
    private static final String TOMS_PARAMETER = "toms";

    @PersistenceContext
    private EntityManager entityManager;

    @Override
    public Optional<Tom> fetchBagRelationships(Optional<Tom> tom) {
        return tom.map(this::fetchSecurityMetrics);
    }

    @Override
    public Page<Tom> fetchBagRelationships(Page<Tom> toms) {
        return new PageImpl<>(fetchBagRelationships(toms.getContent()), toms.getPageable(), toms.getTotalElements());
    }

    @Override
    public List<Tom> fetchBagRelationships(List<Tom> toms) {
        return Optional.of(toms).map(this::fetchSecurityMetrics).orElse(Collections.emptyList());
    }

    Tom fetchSecurityMetrics(Tom result) {
        return entityManager
            .createQuery("select tom from Tom tom left join fetch tom.securityMetrics where tom.id = :id", Tom.class)
            .setParameter(ID_PARAMETER, result.getId())
            .getSingleResult();
    }

    List<Tom> fetchSecurityMetrics(List<Tom> toms) {
        HashMap<Object, Integer> order = new HashMap<>();
        IntStream.range(0, toms.size()).forEach(index -> order.put(toms.get(index).getId(), index));
        List<Tom> result = entityManager
            .createQuery("select tom from Tom tom left join fetch tom.securityMetrics where tom in :toms", Tom.class)
            .setParameter(TOMS_PARAMETER, toms)
            .getResultList();
        Collections.sort(result, (o1, o2) -> Integer.compare(order.get(o1.getId()), order.get(o2.getId())));
        return result;
    }
}
