package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class CloudServiceAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceAllPropertiesEquals(CloudService expected, CloudService actual) {
        assertCloudServiceAutoGeneratedPropertiesEquals(expected, actual);
        assertCloudServiceAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceAllUpdatablePropertiesEquals(CloudService expected, CloudService actual) {
        assertCloudServiceUpdatableFieldsEquals(expected, actual);
        assertCloudServiceUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceAutoGeneratedPropertiesEquals(CloudService expected, CloudService actual) {
        assertThat(expected)
            .as("Verify CloudService auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceUpdatableFieldsEquals(CloudService expected, CloudService actual) {
        assertThat(expected)
            .as("Verify CloudService relevant properties")
            .satisfies(e -> assertThat(e.getAcronym()).as("check acronym").isEqualTo(actual.getAcronym()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getType()).as("check type").isEqualTo(actual.getType()))
            .satisfies(e -> assertThat(e.getDescription()).as("check description").isEqualTo(actual.getDescription()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceUpdatableRelationshipsEquals(CloudService expected, CloudService actual) {
        assertThat(expected)
            .as("Verify CloudService relationships")
            .satisfies(
                e -> assertThat(e.getCloudServiceProvider()).as("check cloudServiceProvider").isEqualTo(actual.getCloudServiceProvider())
            )
            .satisfies(
                e ->
                    assertThat(e.getSecurityControlFramework())
                        .as("check securityControlFramework")
                        .isEqualTo(actual.getSecurityControlFramework())
            );
    }
}
