package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class CloudServiceProviderAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceProviderAllPropertiesEquals(CloudServiceProvider expected, CloudServiceProvider actual) {
        assertCloudServiceProviderAutoGeneratedPropertiesEquals(expected, actual);
        assertCloudServiceProviderAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceProviderAllUpdatablePropertiesEquals(CloudServiceProvider expected, CloudServiceProvider actual) {
        assertCloudServiceProviderUpdatableFieldsEquals(expected, actual);
        assertCloudServiceProviderUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceProviderAutoGeneratedPropertiesEquals(CloudServiceProvider expected, CloudServiceProvider actual) {
        assertThat(expected)
            .as("Verify CloudServiceProvider auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceProviderUpdatableFieldsEquals(CloudServiceProvider expected, CloudServiceProvider actual) {
        assertThat(expected)
            .as("Verify CloudServiceProvider relevant properties")
            .satisfies(e -> assertThat(e.getAcronym()).as("check acronym").isEqualTo(actual.getAcronym()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getContactDetails()).as("check contactDetails").isEqualTo(actual.getContactDetails()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertCloudServiceProviderUpdatableRelationshipsEquals(CloudServiceProvider expected, CloudServiceProvider actual) {}
}
