package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.CloudServiceProviderTestSamples.*;
import static com.emerald.rcm.backend.domain.CloudServiceTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class CloudServiceProviderTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(CloudServiceProvider.class);
        CloudServiceProvider cloudServiceProvider1 = getCloudServiceProviderSample1();
        CloudServiceProvider cloudServiceProvider2 = new CloudServiceProvider();
        assertThat(cloudServiceProvider1).isNotEqualTo(cloudServiceProvider2);

        cloudServiceProvider2.setId(cloudServiceProvider1.getId());
        assertThat(cloudServiceProvider1).isEqualTo(cloudServiceProvider2);

        cloudServiceProvider2 = getCloudServiceProviderSample2();
        assertThat(cloudServiceProvider1).isNotEqualTo(cloudServiceProvider2);
    }

    @Test
    void cloudServiceTest() {
        CloudServiceProvider cloudServiceProvider = getCloudServiceProviderRandomSampleGenerator();
        CloudService cloudServiceBack = getCloudServiceRandomSampleGenerator();

        cloudServiceProvider.addCloudService(cloudServiceBack);
        assertThat(cloudServiceProvider.getCloudServices()).containsOnly(cloudServiceBack);
        assertThat(cloudServiceBack.getCloudServiceProvider()).isEqualTo(cloudServiceProvider);

        cloudServiceProvider.removeCloudService(cloudServiceBack);
        assertThat(cloudServiceProvider.getCloudServices()).doesNotContain(cloudServiceBack);
        assertThat(cloudServiceBack.getCloudServiceProvider()).isNull();

        cloudServiceProvider.cloudServices(new HashSet<>(Set.of(cloudServiceBack)));
        assertThat(cloudServiceProvider.getCloudServices()).containsOnly(cloudServiceBack);
        assertThat(cloudServiceBack.getCloudServiceProvider()).isEqualTo(cloudServiceProvider);

        cloudServiceProvider.setCloudServices(new HashSet<>());
        assertThat(cloudServiceProvider.getCloudServices()).doesNotContain(cloudServiceBack);
        assertThat(cloudServiceBack.getCloudServiceProvider()).isNull();
    }
}
