package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.CloudServiceProviderTestSamples.*;
import static com.emerald.rcm.backend.domain.CloudServiceTestSamples.*;
import static com.emerald.rcm.backend.domain.ResourceTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityControlFrameworkTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class CloudServiceTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(CloudService.class);
        CloudService cloudService1 = getCloudServiceSample1();
        CloudService cloudService2 = new CloudService();
        assertThat(cloudService1).isNotEqualTo(cloudService2);

        cloudService2.setId(cloudService1.getId());
        assertThat(cloudService1).isEqualTo(cloudService2);

        cloudService2 = getCloudServiceSample2();
        assertThat(cloudService1).isNotEqualTo(cloudService2);
    }

    @Test
    void resourceTest() {
        CloudService cloudService = getCloudServiceRandomSampleGenerator();
        Resource resourceBack = getResourceRandomSampleGenerator();

        cloudService.addResource(resourceBack);
        assertThat(cloudService.getResources()).containsOnly(resourceBack);
        assertThat(resourceBack.getCloudService()).isEqualTo(cloudService);

        cloudService.removeResource(resourceBack);
        assertThat(cloudService.getResources()).doesNotContain(resourceBack);
        assertThat(resourceBack.getCloudService()).isNull();

        cloudService.resources(new HashSet<>(Set.of(resourceBack)));
        assertThat(cloudService.getResources()).containsOnly(resourceBack);
        assertThat(resourceBack.getCloudService()).isEqualTo(cloudService);

        cloudService.setResources(new HashSet<>());
        assertThat(cloudService.getResources()).doesNotContain(resourceBack);
        assertThat(resourceBack.getCloudService()).isNull();
    }

    @Test
    void cloudServiceProviderTest() {
        CloudService cloudService = getCloudServiceRandomSampleGenerator();
        CloudServiceProvider cloudServiceProviderBack = getCloudServiceProviderRandomSampleGenerator();

        cloudService.setCloudServiceProvider(cloudServiceProviderBack);
        assertThat(cloudService.getCloudServiceProvider()).isEqualTo(cloudServiceProviderBack);

        cloudService.cloudServiceProvider(null);
        assertThat(cloudService.getCloudServiceProvider()).isNull();
    }

    @Test
    void securityControlFrameworkTest() {
        CloudService cloudService = getCloudServiceRandomSampleGenerator();
        SecurityControlFramework securityControlFrameworkBack = getSecurityControlFrameworkRandomSampleGenerator();

        cloudService.setSecurityControlFramework(securityControlFrameworkBack);
        assertThat(cloudService.getSecurityControlFramework()).isEqualTo(securityControlFrameworkBack);

        cloudService.securityControlFramework(null);
        assertThat(cloudService.getSecurityControlFramework()).isNull();
    }
}
