package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class QuestionAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionAllPropertiesEquals(Question expected, Question actual) {
        assertQuestionAutoGeneratedPropertiesEquals(expected, actual);
        assertQuestionAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionAllUpdatablePropertiesEquals(Question expected, Question actual) {
        assertQuestionUpdatableFieldsEquals(expected, actual);
        assertQuestionUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionAutoGeneratedPropertiesEquals(Question expected, Question actual) {
        assertThat(expected)
            .as("Verify Question auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionUpdatableFieldsEquals(Question expected, Question actual) {
        assertThat(expected)
            .as("Verify Question relevant properties")
            .satisfies(e -> assertThat(e.getCode()).as("check code").isEqualTo(actual.getCode()))
            .satisfies(e -> assertThat(e.getQuestion()).as("check question").isEqualTo(actual.getQuestion()))
            .satisfies(e -> assertThat(e.getDefaultEvidence()).as("check defaultEvidence").isEqualTo(actual.getDefaultEvidence()))
            .satisfies(e -> assertThat(e.getDefaultComment()).as("check defaultComment").isEqualTo(actual.getDefaultComment()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionUpdatableRelationshipsEquals(Question expected, Question actual) {
        assertThat(expected)
            .as("Verify Question relationships")
            .satisfies(
                e ->
                    assertThat(e.getQuestionAssuranceLevel())
                        .as("check questionAssuranceLevel")
                        .isEqualTo(actual.getQuestionAssuranceLevel())
            )
            .satisfies(e -> assertThat(e.getSecurityControl()).as("check securityControl").isEqualTo(actual.getSecurityControl()))
            .satisfies(e -> assertThat(e.getTom()).as("check tom").isEqualTo(actual.getTom()))
            .satisfies(
                e ->
                    assertThat(e.getSecurityControlFramework())
                        .as("check securityControlFramework")
                        .isEqualTo(actual.getSecurityControlFramework())
            );
    }
}
