package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class QuestionnaireAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionnaireAllPropertiesEquals(Questionnaire expected, Questionnaire actual) {
        assertQuestionnaireAutoGeneratedPropertiesEquals(expected, actual);
        assertQuestionnaireAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionnaireAllUpdatablePropertiesEquals(Questionnaire expected, Questionnaire actual) {
        assertQuestionnaireUpdatableFieldsEquals(expected, actual);
        assertQuestionnaireUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionnaireAutoGeneratedPropertiesEquals(Questionnaire expected, Questionnaire actual) {
        assertThat(expected)
            .as("Verify Questionnaire auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionnaireUpdatableFieldsEquals(Questionnaire expected, Questionnaire actual) {
        assertThat(expected)
            .as("Verify Questionnaire relevant properties")
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getEvidences()).as("check evidences").isEqualTo(actual.getEvidences()))
            .satisfies(e -> assertThat(e.getComments()).as("check comments").isEqualTo(actual.getComments()))
            .satisfies(e -> assertThat(e.getLastUpdate()).as("check lastUpdate").isEqualTo(actual.getLastUpdate()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertQuestionnaireUpdatableRelationshipsEquals(Questionnaire expected, Questionnaire actual) {
        assertThat(expected)
            .as("Verify Questionnaire relationships")
            .satisfies(
                e ->
                    assertThat(e.getSecurityControlFramework())
                        .as("check securityControlFramework")
                        .isEqualTo(actual.getSecurityControlFramework())
            )
            .satisfies(e -> assertThat(e.getSecurityControl()).as("check securityControl").isEqualTo(actual.getSecurityControl()))
            .satisfies(
                e ->
                    assertThat(e.getQuestionAssuranceLevel())
                        .as("check questionAssuranceLevel")
                        .isEqualTo(actual.getQuestionAssuranceLevel())
            )
            .satisfies(e -> assertThat(e.getQuestion()).as("check question").isEqualTo(actual.getQuestion()))
            .satisfies(e -> assertThat(e.getQuestionAnswer()).as("check questionAnswer").isEqualTo(actual.getQuestionAnswer()));
    }
}
