package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class ReferenceTomAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertReferenceTomAllPropertiesEquals(ReferenceTom expected, ReferenceTom actual) {
        assertReferenceTomAutoGeneratedPropertiesEquals(expected, actual);
        assertReferenceTomAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertReferenceTomAllUpdatablePropertiesEquals(ReferenceTom expected, ReferenceTom actual) {
        assertReferenceTomUpdatableFieldsEquals(expected, actual);
        assertReferenceTomUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertReferenceTomAutoGeneratedPropertiesEquals(ReferenceTom expected, ReferenceTom actual) {
        assertThat(expected)
            .as("Verify ReferenceTom auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertReferenceTomUpdatableFieldsEquals(ReferenceTom expected, ReferenceTom actual) {
        assertThat(expected)
            .as("Verify ReferenceTom relevant properties")
            .satisfies(e -> assertThat(e.getReferenceTomImpl()).as("check referenceTomImpl").isEqualTo(actual.getReferenceTomImpl()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertReferenceTomUpdatableRelationshipsEquals(ReferenceTom expected, ReferenceTom actual) {
        assertThat(expected)
            .as("Verify ReferenceTom relationships")
            .satisfies(e -> assertThat(e.getTom()).as("check tom").isEqualTo(actual.getTom()));
    }
}
