package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class ResourceAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceAllPropertiesEquals(Resource expected, Resource actual) {
        assertResourceAutoGeneratedPropertiesEquals(expected, actual);
        assertResourceAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceAllUpdatablePropertiesEquals(Resource expected, Resource actual) {
        assertResourceUpdatableFieldsEquals(expected, actual);
        assertResourceUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceAutoGeneratedPropertiesEquals(Resource expected, Resource actual) {
        assertThat(expected)
            .as("Verify Resource auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceUpdatableFieldsEquals(Resource expected, Resource actual) {
        assertThat(expected)
            .as("Verify Resource relevant properties")
            .satisfies(e -> assertThat(e.getAcronym()).as("check acronym").isEqualTo(actual.getAcronym()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(
                e -> assertThat(e.getConfidentialityRisk()).as("check confidentialityRisk").isEqualTo(actual.getConfidentialityRisk())
            )
            .satisfies(e -> assertThat(e.getAvailabilityRisk()).as("check availabilityRisk").isEqualTo(actual.getAvailabilityRisk()))
            .satisfies(e -> assertThat(e.getIntegrityRisk()).as("check integrityRisk").isEqualTo(actual.getIntegrityRisk()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceUpdatableRelationshipsEquals(Resource expected, Resource actual) {
        assertThat(expected)
            .as("Verify Resource relationships")
            .satisfies(e -> assertThat(e.getTargetValue()).as("check targetValue").isEqualTo(actual.getTargetValue()))
            .satisfies(e -> assertThat(e.getResourceType()).as("check resourceType").isEqualTo(actual.getResourceType()))
            .satisfies(e -> assertThat(e.getCloudService()).as("check cloudService").isEqualTo(actual.getCloudService()))
            .satisfies(e -> assertThat(e.getSecurityControls()).as("check securityControls").isEqualTo(actual.getSecurityControls()));
    }
}
