package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.CloudServiceTestSamples.*;
import static com.emerald.rcm.backend.domain.ResourceTestSamples.*;
import static com.emerald.rcm.backend.domain.ResourceTypeTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityControlTestSamples.*;
import static com.emerald.rcm.backend.domain.TargetValueTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class ResourceTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(Resource.class);
        Resource resource1 = getResourceSample1();
        Resource resource2 = new Resource();
        assertThat(resource1).isNotEqualTo(resource2);

        resource2.setId(resource1.getId());
        assertThat(resource1).isEqualTo(resource2);

        resource2 = getResourceSample2();
        assertThat(resource1).isNotEqualTo(resource2);
    }

    @Test
    void targetValueTest() {
        Resource resource = getResourceRandomSampleGenerator();
        TargetValue targetValueBack = getTargetValueRandomSampleGenerator();

        resource.setTargetValue(targetValueBack);
        assertThat(resource.getTargetValue()).isEqualTo(targetValueBack);

        resource.targetValue(null);
        assertThat(resource.getTargetValue()).isNull();
    }

    @Test
    void resourceTypeTest() {
        Resource resource = getResourceRandomSampleGenerator();
        ResourceType resourceTypeBack = getResourceTypeRandomSampleGenerator();

        resource.setResourceType(resourceTypeBack);
        assertThat(resource.getResourceType()).isEqualTo(resourceTypeBack);

        resource.resourceType(null);
        assertThat(resource.getResourceType()).isNull();
    }

    @Test
    void cloudServiceTest() {
        Resource resource = getResourceRandomSampleGenerator();
        CloudService cloudServiceBack = getCloudServiceRandomSampleGenerator();

        resource.setCloudService(cloudServiceBack);
        assertThat(resource.getCloudService()).isEqualTo(cloudServiceBack);

        resource.cloudService(null);
        assertThat(resource.getCloudService()).isNull();
    }

    @Test
    void securityControlTest() {
        Resource resource = getResourceRandomSampleGenerator();
        SecurityControl securityControlBack = getSecurityControlRandomSampleGenerator();

        resource.addSecurityControl(securityControlBack);
        assertThat(resource.getSecurityControls()).containsOnly(securityControlBack);
        assertThat(securityControlBack.getResources()).containsOnly(resource);

        resource.removeSecurityControl(securityControlBack);
        assertThat(resource.getSecurityControls()).doesNotContain(securityControlBack);
        assertThat(securityControlBack.getResources()).doesNotContain(resource);

        resource.securityControls(new HashSet<>(Set.of(securityControlBack)));
        assertThat(resource.getSecurityControls()).containsOnly(securityControlBack);
        assertThat(securityControlBack.getResources()).containsOnly(resource);

        resource.setSecurityControls(new HashSet<>());
        assertThat(resource.getSecurityControls()).doesNotContain(securityControlBack);
        assertThat(securityControlBack.getResources()).doesNotContain(resource);
    }
}
