package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class ResourceTypeAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceTypeAllPropertiesEquals(ResourceType expected, ResourceType actual) {
        assertResourceTypeAutoGeneratedPropertiesEquals(expected, actual);
        assertResourceTypeAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceTypeAllUpdatablePropertiesEquals(ResourceType expected, ResourceType actual) {
        assertResourceTypeUpdatableFieldsEquals(expected, actual);
        assertResourceTypeUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceTypeAutoGeneratedPropertiesEquals(ResourceType expected, ResourceType actual) {
        assertThat(expected)
            .as("Verify ResourceType auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceTypeUpdatableFieldsEquals(ResourceType expected, ResourceType actual) {
        assertThat(expected)
            .as("Verify ResourceType relevant properties")
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getOntologyTerm()).as("check ontologyTerm").isEqualTo(actual.getOntologyTerm()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertResourceTypeUpdatableRelationshipsEquals(ResourceType expected, ResourceType actual) {}
}
