package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class SecurityControlAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlAllPropertiesEquals(SecurityControl expected, SecurityControl actual) {
        assertSecurityControlAutoGeneratedPropertiesEquals(expected, actual);
        assertSecurityControlAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlAllUpdatablePropertiesEquals(SecurityControl expected, SecurityControl actual) {
        assertSecurityControlUpdatableFieldsEquals(expected, actual);
        assertSecurityControlUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlAutoGeneratedPropertiesEquals(SecurityControl expected, SecurityControl actual) {
        assertThat(expected)
            .as("Verify SecurityControl auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlUpdatableFieldsEquals(SecurityControl expected, SecurityControl actual) {
        assertThat(expected)
            .as("Verify SecurityControl relevant properties")
            .satisfies(e -> assertThat(e.getCode()).as("check code").isEqualTo(actual.getCode()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getObjective()).as("check objective").isEqualTo(actual.getObjective()))
            .satisfies(e -> assertThat(e.getDescription()).as("check description").isEqualTo(actual.getDescription()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlUpdatableRelationshipsEquals(SecurityControl expected, SecurityControl actual) {
        assertThat(expected)
            .as("Verify SecurityControl relationships")
            .satisfies(
                e ->
                    assertThat(e.getSecurityControlCategory())
                        .as("check securityControlCategory")
                        .isEqualTo(actual.getSecurityControlCategory())
            )
            .satisfies(e -> assertThat(e.getResources()).as("check resources").isEqualTo(actual.getResources()));
    }
}
