package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class SecurityControlCategoryAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlCategoryAllPropertiesEquals(SecurityControlCategory expected, SecurityControlCategory actual) {
        assertSecurityControlCategoryAutoGeneratedPropertiesEquals(expected, actual);
        assertSecurityControlCategoryAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlCategoryAllUpdatablePropertiesEquals(
        SecurityControlCategory expected,
        SecurityControlCategory actual
    ) {
        assertSecurityControlCategoryUpdatableFieldsEquals(expected, actual);
        assertSecurityControlCategoryUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlCategoryAutoGeneratedPropertiesEquals(
        SecurityControlCategory expected,
        SecurityControlCategory actual
    ) {
        assertThat(expected)
            .as("Verify SecurityControlCategory auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlCategoryUpdatableFieldsEquals(
        SecurityControlCategory expected,
        SecurityControlCategory actual
    ) {
        assertThat(expected)
            .as("Verify SecurityControlCategory relevant properties")
            .satisfies(e -> assertThat(e.getCode()).as("check code").isEqualTo(actual.getCode()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getDescription()).as("check description").isEqualTo(actual.getDescription()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlCategoryUpdatableRelationshipsEquals(
        SecurityControlCategory expected,
        SecurityControlCategory actual
    ) {
        assertThat(expected)
            .as("Verify SecurityControlCategory relationships")
            .satisfies(
                e ->
                    assertThat(e.getSecurityControlFramework())
                        .as("check securityControlFramework")
                        .isEqualTo(actual.getSecurityControlFramework())
            );
    }
}
