package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class SecurityControlFrameworkAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlFrameworkAllPropertiesEquals(
        SecurityControlFramework expected,
        SecurityControlFramework actual
    ) {
        assertSecurityControlFrameworkAutoGeneratedPropertiesEquals(expected, actual);
        assertSecurityControlFrameworkAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlFrameworkAllUpdatablePropertiesEquals(
        SecurityControlFramework expected,
        SecurityControlFramework actual
    ) {
        assertSecurityControlFrameworkUpdatableFieldsEquals(expected, actual);
        assertSecurityControlFrameworkUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlFrameworkAutoGeneratedPropertiesEquals(
        SecurityControlFramework expected,
        SecurityControlFramework actual
    ) {
        assertThat(expected)
            .as("Verify SecurityControlFramework auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlFrameworkUpdatableFieldsEquals(
        SecurityControlFramework expected,
        SecurityControlFramework actual
    ) {
        assertThat(expected)
            .as("Verify SecurityControlFramework relevant properties")
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getDescription()).as("check description").isEqualTo(actual.getDescription()))
            .satisfies(e -> assertThat(e.getVersion()).as("check version").isEqualTo(actual.getVersion()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityControlFrameworkUpdatableRelationshipsEquals(
        SecurityControlFramework expected,
        SecurityControlFramework actual
    ) {}
}
