package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.CloudServiceTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityControlFrameworkTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class SecurityControlFrameworkTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(SecurityControlFramework.class);
        SecurityControlFramework securityControlFramework1 = getSecurityControlFrameworkSample1();
        SecurityControlFramework securityControlFramework2 = new SecurityControlFramework();
        assertThat(securityControlFramework1).isNotEqualTo(securityControlFramework2);

        securityControlFramework2.setId(securityControlFramework1.getId());
        assertThat(securityControlFramework1).isEqualTo(securityControlFramework2);

        securityControlFramework2 = getSecurityControlFrameworkSample2();
        assertThat(securityControlFramework1).isNotEqualTo(securityControlFramework2);
    }

    @Test
    void cloudServiceTest() {
        SecurityControlFramework securityControlFramework = getSecurityControlFrameworkRandomSampleGenerator();
        CloudService cloudServiceBack = getCloudServiceRandomSampleGenerator();

        securityControlFramework.addCloudService(cloudServiceBack);
        assertThat(securityControlFramework.getCloudServices()).containsOnly(cloudServiceBack);
        assertThat(cloudServiceBack.getSecurityControlFramework()).isEqualTo(securityControlFramework);

        securityControlFramework.removeCloudService(cloudServiceBack);
        assertThat(securityControlFramework.getCloudServices()).doesNotContain(cloudServiceBack);
        assertThat(cloudServiceBack.getSecurityControlFramework()).isNull();

        securityControlFramework.cloudServices(new HashSet<>(Set.of(cloudServiceBack)));
        assertThat(securityControlFramework.getCloudServices()).containsOnly(cloudServiceBack);
        assertThat(cloudServiceBack.getSecurityControlFramework()).isEqualTo(securityControlFramework);

        securityControlFramework.setCloudServices(new HashSet<>());
        assertThat(securityControlFramework.getCloudServices()).doesNotContain(cloudServiceBack);
        assertThat(cloudServiceBack.getSecurityControlFramework()).isNull();
    }
}
