package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.ResourceTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityControlCategoryTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityControlTestSamples.*;
import static com.emerald.rcm.backend.domain.SimilarControlTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class SecurityControlTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(SecurityControl.class);
        SecurityControl securityControl1 = getSecurityControlSample1();
        SecurityControl securityControl2 = new SecurityControl();
        assertThat(securityControl1).isNotEqualTo(securityControl2);

        securityControl2.setId(securityControl1.getId());
        assertThat(securityControl1).isEqualTo(securityControl2);

        securityControl2 = getSecurityControlSample2();
        assertThat(securityControl1).isNotEqualTo(securityControl2);
    }

    @Test
    void similarControlsTest() {
        SecurityControl securityControl = getSecurityControlRandomSampleGenerator();
        SimilarControl similarControlBack = getSimilarControlRandomSampleGenerator();

        securityControl.addSimilarControls(similarControlBack);
        assertThat(securityControl.getSimilarControls()).containsOnly(similarControlBack);
        assertThat(similarControlBack.getSecurityControl()).isEqualTo(securityControl);

        securityControl.removeSimilarControls(similarControlBack);
        assertThat(securityControl.getSimilarControls()).doesNotContain(similarControlBack);
        assertThat(similarControlBack.getSecurityControl()).isNull();

        securityControl.similarControls(new HashSet<>(Set.of(similarControlBack)));
        assertThat(securityControl.getSimilarControls()).containsOnly(similarControlBack);
        assertThat(similarControlBack.getSecurityControl()).isEqualTo(securityControl);

        securityControl.setSimilarControls(new HashSet<>());
        assertThat(securityControl.getSimilarControls()).doesNotContain(similarControlBack);
        assertThat(similarControlBack.getSecurityControl()).isNull();
    }

    @Test
    void securityControlCategoryTest() {
        SecurityControl securityControl = getSecurityControlRandomSampleGenerator();
        SecurityControlCategory securityControlCategoryBack = getSecurityControlCategoryRandomSampleGenerator();

        securityControl.setSecurityControlCategory(securityControlCategoryBack);
        assertThat(securityControl.getSecurityControlCategory()).isEqualTo(securityControlCategoryBack);

        securityControl.securityControlCategory(null);
        assertThat(securityControl.getSecurityControlCategory()).isNull();
    }

    @Test
    void resourceTest() {
        SecurityControl securityControl = getSecurityControlRandomSampleGenerator();
        Resource resourceBack = getResourceRandomSampleGenerator();

        securityControl.addResource(resourceBack);
        assertThat(securityControl.getResources()).containsOnly(resourceBack);

        securityControl.removeResource(resourceBack);
        assertThat(securityControl.getResources()).doesNotContain(resourceBack);

        securityControl.resources(new HashSet<>(Set.of(resourceBack)));
        assertThat(securityControl.getResources()).containsOnly(resourceBack);

        securityControl.setResources(new HashSet<>());
        assertThat(securityControl.getResources()).doesNotContain(resourceBack);
    }
}
