package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class SecurityMetricAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityMetricAllPropertiesEquals(SecurityMetric expected, SecurityMetric actual) {
        assertSecurityMetricAutoGeneratedPropertiesEquals(expected, actual);
        assertSecurityMetricAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityMetricAllUpdatablePropertiesEquals(SecurityMetric expected, SecurityMetric actual) {
        assertSecurityMetricUpdatableFieldsEquals(expected, actual);
        assertSecurityMetricUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityMetricAutoGeneratedPropertiesEquals(SecurityMetric expected, SecurityMetric actual) {
        assertThat(expected)
            .as("Verify SecurityMetric auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityMetricUpdatableFieldsEquals(SecurityMetric expected, SecurityMetric actual) {
        assertThat(expected)
            .as("Verify SecurityMetric relevant properties")
            .satisfies(e -> assertThat(e.getMetricId()).as("check metricId").isEqualTo(actual.getMetricId()))
            .satisfies(e -> assertThat(e.getCategory()).as("check category").isEqualTo(actual.getCategory()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getSource()).as("check source").isEqualTo(actual.getSource()))
            .satisfies(e -> assertThat(e.getDescription()).as("check description").isEqualTo(actual.getDescription()))
            .satisfies(e -> assertThat(e.getScale()).as("check scale").isEqualTo(actual.getScale()))
            .satisfies(e -> assertThat(e.getOperator()).as("check operator").isEqualTo(actual.getOperator()))
            .satisfies(e -> assertThat(e.getTargetValue()).as("check targetValue").isEqualTo(actual.getTargetValue()))
            .satisfies(
                e -> assertThat(e.getTargetValueDatatype()).as("check targetValueDatatype").isEqualTo(actual.getTargetValueDatatype())
            )
            .satisfies(e -> assertThat(e.getInterval()).as("check interval").isEqualTo(actual.getInterval()))
            .satisfies(e -> assertThat(e.getTargetResourceType()).as("check targetResourceType").isEqualTo(actual.getTargetResourceType()))
            .satisfies(e -> assertThat(e.getSecurityFeature()).as("check securityFeature").isEqualTo(actual.getSecurityFeature()))
            .satisfies(e -> assertThat(e.getKeywords()).as("check keywords").isEqualTo(actual.getKeywords()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSecurityMetricUpdatableRelationshipsEquals(SecurityMetric expected, SecurityMetric actual) {
        assertThat(expected)
            .as("Verify SecurityMetric relationships")
            .satisfies(e -> assertThat(e.getReferenceTom()).as("check referenceTom").isEqualTo(actual.getReferenceTom()))
            .satisfies(e -> assertThat(e.getResourceType()).as("check resourceType").isEqualTo(actual.getResourceType()))
            .satisfies(e -> assertThat(e.getToms()).as("check toms").isEqualTo(actual.getToms()));
    }
}
