package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.ReferenceTomTestSamples.*;
import static com.emerald.rcm.backend.domain.ResourceTypeTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityMetricTestSamples.*;
import static com.emerald.rcm.backend.domain.TomTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class SecurityMetricTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(SecurityMetric.class);
        SecurityMetric securityMetric1 = getSecurityMetricSample1();
        SecurityMetric securityMetric2 = new SecurityMetric();
        assertThat(securityMetric1).isNotEqualTo(securityMetric2);

        securityMetric2.setId(securityMetric1.getId());
        assertThat(securityMetric1).isEqualTo(securityMetric2);

        securityMetric2 = getSecurityMetricSample2();
        assertThat(securityMetric1).isNotEqualTo(securityMetric2);
    }

    @Test
    void referenceTomTest() {
        SecurityMetric securityMetric = getSecurityMetricRandomSampleGenerator();
        ReferenceTom referenceTomBack = getReferenceTomRandomSampleGenerator();

        securityMetric.setReferenceTom(referenceTomBack);
        assertThat(securityMetric.getReferenceTom()).isEqualTo(referenceTomBack);

        securityMetric.referenceTom(null);
        assertThat(securityMetric.getReferenceTom()).isNull();
    }

    @Test
    void resourceTypeTest() {
        SecurityMetric securityMetric = getSecurityMetricRandomSampleGenerator();
        ResourceType resourceTypeBack = getResourceTypeRandomSampleGenerator();

        securityMetric.setResourceType(resourceTypeBack);
        assertThat(securityMetric.getResourceType()).isEqualTo(resourceTypeBack);

        securityMetric.resourceType(null);
        assertThat(securityMetric.getResourceType()).isNull();
    }

    @Test
    void tomTest() {
        SecurityMetric securityMetric = getSecurityMetricRandomSampleGenerator();
        Tom tomBack = getTomRandomSampleGenerator();

        securityMetric.addTom(tomBack);
        assertThat(securityMetric.getToms()).containsOnly(tomBack);
        assertThat(tomBack.getSecurityMetrics()).containsOnly(securityMetric);

        securityMetric.removeTom(tomBack);
        assertThat(securityMetric.getToms()).doesNotContain(tomBack);
        assertThat(tomBack.getSecurityMetrics()).doesNotContain(securityMetric);

        securityMetric.toms(new HashSet<>(Set.of(tomBack)));
        assertThat(securityMetric.getToms()).containsOnly(tomBack);
        assertThat(tomBack.getSecurityMetrics()).containsOnly(securityMetric);

        securityMetric.setToms(new HashSet<>());
        assertThat(securityMetric.getToms()).doesNotContain(tomBack);
        assertThat(tomBack.getSecurityMetrics()).doesNotContain(securityMetric);
    }
}
