package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class SimilarControlAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSimilarControlAllPropertiesEquals(SimilarControl expected, SimilarControl actual) {
        assertSimilarControlAutoGeneratedPropertiesEquals(expected, actual);
        assertSimilarControlAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSimilarControlAllUpdatablePropertiesEquals(SimilarControl expected, SimilarControl actual) {
        assertSimilarControlUpdatableFieldsEquals(expected, actual);
        assertSimilarControlUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSimilarControlAutoGeneratedPropertiesEquals(SimilarControl expected, SimilarControl actual) {
        assertThat(expected)
            .as("Verify SimilarControl auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSimilarControlUpdatableFieldsEquals(SimilarControl expected, SimilarControl actual) {
        assertThat(expected)
            .as("Verify SimilarControl relevant properties")
            .satisfies(e -> assertThat(e.getCode()).as("check code").isEqualTo(actual.getCode()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(
                e ->
                    assertThat(e.getSecurityControlFrameworkName())
                        .as("check securityControlFrameworkName")
                        .isEqualTo(actual.getSecurityControlFrameworkName())
            );
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertSimilarControlUpdatableRelationshipsEquals(SimilarControl expected, SimilarControl actual) {
        assertThat(expected)
            .as("Verify SimilarControl relationships")
            .satisfies(e -> assertThat(e.getSecurityControl()).as("check securityControl").isEqualTo(actual.getSecurityControl()));
    }
}
