package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class TargetValueAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTargetValueAllPropertiesEquals(TargetValue expected, TargetValue actual) {
        assertTargetValueAutoGeneratedPropertiesEquals(expected, actual);
        assertTargetValueAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTargetValueAllUpdatablePropertiesEquals(TargetValue expected, TargetValue actual) {
        assertTargetValueUpdatableFieldsEquals(expected, actual);
        assertTargetValueUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTargetValueAutoGeneratedPropertiesEquals(TargetValue expected, TargetValue actual) {
        assertThat(expected)
            .as("Verify TargetValue auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTargetValueUpdatableFieldsEquals(TargetValue expected, TargetValue actual) {
        assertThat(expected)
            .as("Verify TargetValue relevant properties")
            .satisfies(e -> assertThat(e.getValue()).as("check value").isEqualTo(actual.getValue()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTargetValueUpdatableRelationshipsEquals(TargetValue expected, TargetValue actual) {}
}
