package com.emerald.rcm.backend.domain;

import static org.assertj.core.api.Assertions.assertThat;

public class TomAsserts {

    /**
     * Asserts that the entity has all properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTomAllPropertiesEquals(Tom expected, Tom actual) {
        assertTomAutoGeneratedPropertiesEquals(expected, actual);
        assertTomAllUpdatablePropertiesEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all updatable properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTomAllUpdatablePropertiesEquals(Tom expected, Tom actual) {
        assertTomUpdatableFieldsEquals(expected, actual);
        assertTomUpdatableRelationshipsEquals(expected, actual);
    }

    /**
     * Asserts that the entity has all the auto generated properties (fields/relationships) set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTomAutoGeneratedPropertiesEquals(Tom expected, Tom actual) {
        assertThat(expected)
            .as("Verify Tom auto generated properties")
            .satisfies(e -> assertThat(e.getId()).as("check id").isEqualTo(actual.getId()));
    }

    /**
     * Asserts that the entity has all the updatable fields set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTomUpdatableFieldsEquals(Tom expected, Tom actual) {
        assertThat(expected)
            .as("Verify Tom relevant properties")
            .satisfies(e -> assertThat(e.getCode()).as("check code").isEqualTo(actual.getCode()))
            .satisfies(e -> assertThat(e.getName()).as("check name").isEqualTo(actual.getName()))
            .satisfies(e -> assertThat(e.getDescription()).as("check description").isEqualTo(actual.getDescription()))
            .satisfies(e -> assertThat(e.getAssuranceLevel()).as("check assuranceLevel").isEqualTo(actual.getAssuranceLevel()))
            .satisfies(e -> assertThat(e.getType()).as("check type").isEqualTo(actual.getType()))
            .satisfies(e -> assertThat(e.getGuidance()).as("check guidance").isEqualTo(actual.getGuidance()));
    }

    /**
     * Asserts that the entity has all the updatable relationships set.
     *
     * @param expected the expected entity
     * @param actual the actual entity
     */
    public static void assertTomUpdatableRelationshipsEquals(Tom expected, Tom actual) {
        assertThat(expected)
            .as("Verify Tom relationships")
            .satisfies(e -> assertThat(e.getSecurityControl()).as("check securityControl").isEqualTo(actual.getSecurityControl()))
            .satisfies(e -> assertThat(e.getSecurityMetrics()).as("check securityMetrics").isEqualTo(actual.getSecurityMetrics()));
    }
}
