package com.emerald.rcm.backend.domain;

import static com.emerald.rcm.backend.domain.SecurityControlTestSamples.*;
import static com.emerald.rcm.backend.domain.SecurityMetricTestSamples.*;
import static com.emerald.rcm.backend.domain.TomTestSamples.*;
import static org.assertj.core.api.Assertions.assertThat;

import com.emerald.rcm.backend.web.rest.TestUtil;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.Test;

class TomTest {

    @Test
    void equalsVerifier() throws Exception {
        TestUtil.equalsVerifier(Tom.class);
        Tom tom1 = getTomSample1();
        Tom tom2 = new Tom();
        assertThat(tom1).isNotEqualTo(tom2);

        tom2.setId(tom1.getId());
        assertThat(tom1).isEqualTo(tom2);

        tom2 = getTomSample2();
        assertThat(tom1).isNotEqualTo(tom2);
    }

    @Test
    void securityControlTest() {
        Tom tom = getTomRandomSampleGenerator();
        SecurityControl securityControlBack = getSecurityControlRandomSampleGenerator();

        tom.setSecurityControl(securityControlBack);
        assertThat(tom.getSecurityControl()).isEqualTo(securityControlBack);

        tom.securityControl(null);
        assertThat(tom.getSecurityControl()).isNull();
    }

    @Test
    void securityMetricTest() {
        Tom tom = getTomRandomSampleGenerator();
        SecurityMetric securityMetricBack = getSecurityMetricRandomSampleGenerator();

        tom.addSecurityMetric(securityMetricBack);
        assertThat(tom.getSecurityMetrics()).containsOnly(securityMetricBack);

        tom.removeSecurityMetric(securityMetricBack);
        assertThat(tom.getSecurityMetrics()).doesNotContain(securityMetricBack);

        tom.securityMetrics(new HashSet<>(Set.of(securityMetricBack)));
        assertThat(tom.getSecurityMetrics()).containsOnly(securityMetricBack);

        tom.setSecurityMetrics(new HashSet<>());
        assertThat(tom.getSecurityMetrics()).doesNotContain(securityMetricBack);
    }
}
