package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.CloudServiceProviderAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.CloudServiceProvider;
import com.emerald.rcm.backend.repository.CloudServiceProviderRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link CloudServiceProviderResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@AutoConfigureMockMvc
@WithMockUser
class CloudServiceProviderResourceIT {

    private static final String DEFAULT_ACRONYM = "AAAAAAAAAA";
    private static final String UPDATED_ACRONYM = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_CONTACT_DETAILS = "AAAAAAAAAA";
    private static final String UPDATED_CONTACT_DETAILS = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/cloud-service-providers";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private CloudServiceProviderRepository cloudServiceProviderRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restCloudServiceProviderMockMvc;

    private CloudServiceProvider cloudServiceProvider;

    private CloudServiceProvider insertedCloudServiceProvider;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static CloudServiceProvider createEntity(EntityManager em) {
        CloudServiceProvider cloudServiceProvider = new CloudServiceProvider()
            .acronym(DEFAULT_ACRONYM)
            .name(DEFAULT_NAME)
            .contactDetails(DEFAULT_CONTACT_DETAILS);
        return cloudServiceProvider;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static CloudServiceProvider createUpdatedEntity(EntityManager em) {
        CloudServiceProvider cloudServiceProvider = new CloudServiceProvider()
            .acronym(UPDATED_ACRONYM)
            .name(UPDATED_NAME)
            .contactDetails(UPDATED_CONTACT_DETAILS);
        return cloudServiceProvider;
    }

    @BeforeEach
    public void initTest() {
        cloudServiceProvider = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedCloudServiceProvider != null) {
            cloudServiceProviderRepository.delete(insertedCloudServiceProvider);
            insertedCloudServiceProvider = null;
        }
    }

    @Test
    @Transactional
    void createCloudServiceProvider() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the CloudServiceProvider
        var returnedCloudServiceProvider = om.readValue(
            restCloudServiceProviderMockMvc
                .perform(
                    post(ENTITY_API_URL)
                        .with(csrf())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(om.writeValueAsBytes(cloudServiceProvider))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            CloudServiceProvider.class
        );

        // Validate the CloudServiceProvider in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertCloudServiceProviderUpdatableFieldsEquals(
            returnedCloudServiceProvider,
            getPersistedCloudServiceProvider(returnedCloudServiceProvider)
        );

        insertedCloudServiceProvider = returnedCloudServiceProvider;
    }

    @Test
    @Transactional
    void createCloudServiceProviderWithExistingId() throws Exception {
        // Create the CloudServiceProvider with an existing ID
        cloudServiceProvider.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restCloudServiceProviderMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkAcronymIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        cloudServiceProvider.setAcronym(null);

        // Create the CloudServiceProvider, which fails.

        restCloudServiceProviderMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        cloudServiceProvider.setName(null);

        // Create the CloudServiceProvider, which fails.

        restCloudServiceProviderMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllCloudServiceProviders() throws Exception {
        // Initialize the database
        insertedCloudServiceProvider = cloudServiceProviderRepository.saveAndFlush(cloudServiceProvider);

        // Get all the cloudServiceProviderList
        restCloudServiceProviderMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(cloudServiceProvider.getId().intValue())))
            .andExpect(jsonPath("$.[*].acronym").value(hasItem(DEFAULT_ACRONYM)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].contactDetails").value(hasItem(DEFAULT_CONTACT_DETAILS.toString())));
    }

    @Test
    @Transactional
    void getCloudServiceProvider() throws Exception {
        // Initialize the database
        insertedCloudServiceProvider = cloudServiceProviderRepository.saveAndFlush(cloudServiceProvider);

        // Get the cloudServiceProvider
        restCloudServiceProviderMockMvc
            .perform(get(ENTITY_API_URL_ID, cloudServiceProvider.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(cloudServiceProvider.getId().intValue()))
            .andExpect(jsonPath("$.acronym").value(DEFAULT_ACRONYM))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.contactDetails").value(DEFAULT_CONTACT_DETAILS.toString()));
    }

    @Test
    @Transactional
    void getNonExistingCloudServiceProvider() throws Exception {
        // Get the cloudServiceProvider
        restCloudServiceProviderMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingCloudServiceProvider() throws Exception {
        // Initialize the database
        insertedCloudServiceProvider = cloudServiceProviderRepository.saveAndFlush(cloudServiceProvider);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the cloudServiceProvider
        CloudServiceProvider updatedCloudServiceProvider = cloudServiceProviderRepository
            .findById(cloudServiceProvider.getId())
            .orElseThrow();
        // Disconnect from session so that the updates on updatedCloudServiceProvider are not directly saved in db
        em.detach(updatedCloudServiceProvider);
        updatedCloudServiceProvider.acronym(UPDATED_ACRONYM).name(UPDATED_NAME).contactDetails(UPDATED_CONTACT_DETAILS);

        restCloudServiceProviderMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedCloudServiceProvider.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedCloudServiceProvider))
            )
            .andExpect(status().isOk());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedCloudServiceProviderToMatchAllProperties(updatedCloudServiceProvider);
    }

    @Test
    @Transactional
    void putNonExistingCloudServiceProvider() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudServiceProvider.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restCloudServiceProviderMockMvc
            .perform(
                put(ENTITY_API_URL_ID, cloudServiceProvider.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchCloudServiceProvider() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudServiceProvider.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceProviderMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamCloudServiceProvider() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudServiceProvider.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceProviderMockMvc
            .perform(
                put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateCloudServiceProviderWithPatch() throws Exception {
        // Initialize the database
        insertedCloudServiceProvider = cloudServiceProviderRepository.saveAndFlush(cloudServiceProvider);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the cloudServiceProvider using partial update
        CloudServiceProvider partialUpdatedCloudServiceProvider = new CloudServiceProvider();
        partialUpdatedCloudServiceProvider.setId(cloudServiceProvider.getId());

        partialUpdatedCloudServiceProvider.name(UPDATED_NAME).contactDetails(UPDATED_CONTACT_DETAILS);

        restCloudServiceProviderMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedCloudServiceProvider.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedCloudServiceProvider))
            )
            .andExpect(status().isOk());

        // Validate the CloudServiceProvider in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertCloudServiceProviderUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedCloudServiceProvider, cloudServiceProvider),
            getPersistedCloudServiceProvider(cloudServiceProvider)
        );
    }

    @Test
    @Transactional
    void fullUpdateCloudServiceProviderWithPatch() throws Exception {
        // Initialize the database
        insertedCloudServiceProvider = cloudServiceProviderRepository.saveAndFlush(cloudServiceProvider);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the cloudServiceProvider using partial update
        CloudServiceProvider partialUpdatedCloudServiceProvider = new CloudServiceProvider();
        partialUpdatedCloudServiceProvider.setId(cloudServiceProvider.getId());

        partialUpdatedCloudServiceProvider.acronym(UPDATED_ACRONYM).name(UPDATED_NAME).contactDetails(UPDATED_CONTACT_DETAILS);

        restCloudServiceProviderMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedCloudServiceProvider.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedCloudServiceProvider))
            )
            .andExpect(status().isOk());

        // Validate the CloudServiceProvider in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertCloudServiceProviderUpdatableFieldsEquals(
            partialUpdatedCloudServiceProvider,
            getPersistedCloudServiceProvider(partialUpdatedCloudServiceProvider)
        );
    }

    @Test
    @Transactional
    void patchNonExistingCloudServiceProvider() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudServiceProvider.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restCloudServiceProviderMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, cloudServiceProvider.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchCloudServiceProvider() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudServiceProvider.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceProviderMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamCloudServiceProvider() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudServiceProvider.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceProviderMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(cloudServiceProvider))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the CloudServiceProvider in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteCloudServiceProvider() throws Exception {
        // Initialize the database
        insertedCloudServiceProvider = cloudServiceProviderRepository.saveAndFlush(cloudServiceProvider);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the cloudServiceProvider
        restCloudServiceProviderMockMvc
            .perform(delete(ENTITY_API_URL_ID, cloudServiceProvider.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return cloudServiceProviderRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected CloudServiceProvider getPersistedCloudServiceProvider(CloudServiceProvider cloudServiceProvider) {
        return cloudServiceProviderRepository.findById(cloudServiceProvider.getId()).orElseThrow();
    }

    protected void assertPersistedCloudServiceProviderToMatchAllProperties(CloudServiceProvider expectedCloudServiceProvider) {
        assertCloudServiceProviderAllPropertiesEquals(
            expectedCloudServiceProvider,
            getPersistedCloudServiceProvider(expectedCloudServiceProvider)
        );
    }

    protected void assertPersistedCloudServiceProviderToMatchUpdatableProperties(CloudServiceProvider expectedCloudServiceProvider) {
        assertCloudServiceProviderAllUpdatablePropertiesEquals(
            expectedCloudServiceProvider,
            getPersistedCloudServiceProvider(expectedCloudServiceProvider)
        );
    }
}
