package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.CloudServiceAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.CloudService;
import com.emerald.rcm.backend.repository.CloudServiceRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link CloudServiceResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class CloudServiceResourceIT {

    private static final String DEFAULT_ACRONYM = "AAAAAAAAAA";
    private static final String UPDATED_ACRONYM = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_TYPE = "AAAAAAAAAA";
    private static final String UPDATED_TYPE = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/cloud-services";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private CloudServiceRepository cloudServiceRepository;

    @Mock
    private CloudServiceRepository cloudServiceRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restCloudServiceMockMvc;

    private CloudService cloudService;

    private CloudService insertedCloudService;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static CloudService createEntity(EntityManager em) {
        CloudService cloudService = new CloudService()
            .acronym(DEFAULT_ACRONYM)
            .name(DEFAULT_NAME)
            .type(DEFAULT_TYPE)
            .description(DEFAULT_DESCRIPTION);
        return cloudService;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static CloudService createUpdatedEntity(EntityManager em) {
        CloudService cloudService = new CloudService()
            .acronym(UPDATED_ACRONYM)
            .name(UPDATED_NAME)
            .type(UPDATED_TYPE)
            .description(UPDATED_DESCRIPTION);
        return cloudService;
    }

    @BeforeEach
    public void initTest() {
        cloudService = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedCloudService != null) {
            cloudServiceRepository.delete(insertedCloudService);
            insertedCloudService = null;
        }
    }

    @Test
    @Transactional
    void createCloudService() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the CloudService
        var returnedCloudService = om.readValue(
            restCloudServiceMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudService))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            CloudService.class
        );

        // Validate the CloudService in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertCloudServiceUpdatableFieldsEquals(returnedCloudService, getPersistedCloudService(returnedCloudService));

        insertedCloudService = returnedCloudService;
    }

    @Test
    @Transactional
    void createCloudServiceWithExistingId() throws Exception {
        // Create the CloudService with an existing ID
        cloudService.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restCloudServiceMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudService)))
            .andExpect(status().isBadRequest());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkAcronymIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        cloudService.setAcronym(null);

        // Create the CloudService, which fails.

        restCloudServiceMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudService)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        cloudService.setName(null);

        // Create the CloudService, which fails.

        restCloudServiceMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudService)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkTypeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        cloudService.setType(null);

        // Create the CloudService, which fails.

        restCloudServiceMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudService)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllCloudServices() throws Exception {
        // Initialize the database
        insertedCloudService = cloudServiceRepository.saveAndFlush(cloudService);

        // Get all the cloudServiceList
        restCloudServiceMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(cloudService.getId().intValue())))
            .andExpect(jsonPath("$.[*].acronym").value(hasItem(DEFAULT_ACRONYM)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].type").value(hasItem(DEFAULT_TYPE)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION.toString())));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllCloudServicesWithEagerRelationshipsIsEnabled() throws Exception {
        when(cloudServiceRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restCloudServiceMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(cloudServiceRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllCloudServicesWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(cloudServiceRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restCloudServiceMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(cloudServiceRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getCloudService() throws Exception {
        // Initialize the database
        insertedCloudService = cloudServiceRepository.saveAndFlush(cloudService);

        // Get the cloudService
        restCloudServiceMockMvc
            .perform(get(ENTITY_API_URL_ID, cloudService.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(cloudService.getId().intValue()))
            .andExpect(jsonPath("$.acronym").value(DEFAULT_ACRONYM))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.type").value(DEFAULT_TYPE))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION.toString()));
    }

    @Test
    @Transactional
    void getNonExistingCloudService() throws Exception {
        // Get the cloudService
        restCloudServiceMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingCloudService() throws Exception {
        // Initialize the database
        insertedCloudService = cloudServiceRepository.saveAndFlush(cloudService);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the cloudService
        CloudService updatedCloudService = cloudServiceRepository.findById(cloudService.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedCloudService are not directly saved in db
        em.detach(updatedCloudService);
        updatedCloudService.acronym(UPDATED_ACRONYM).name(UPDATED_NAME).type(UPDATED_TYPE).description(UPDATED_DESCRIPTION);

        restCloudServiceMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedCloudService.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedCloudService))
            )
            .andExpect(status().isOk());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedCloudServiceToMatchAllProperties(updatedCloudService);
    }

    @Test
    @Transactional
    void putNonExistingCloudService() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudService.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restCloudServiceMockMvc
            .perform(
                put(ENTITY_API_URL_ID, cloudService.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudService))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchCloudService() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudService.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(cloudService))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamCloudService() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudService.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(cloudService)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateCloudServiceWithPatch() throws Exception {
        // Initialize the database
        insertedCloudService = cloudServiceRepository.saveAndFlush(cloudService);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the cloudService using partial update
        CloudService partialUpdatedCloudService = new CloudService();
        partialUpdatedCloudService.setId(cloudService.getId());

        partialUpdatedCloudService.type(UPDATED_TYPE);

        restCloudServiceMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedCloudService.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedCloudService))
            )
            .andExpect(status().isOk());

        // Validate the CloudService in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertCloudServiceUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedCloudService, cloudService),
            getPersistedCloudService(cloudService)
        );
    }

    @Test
    @Transactional
    void fullUpdateCloudServiceWithPatch() throws Exception {
        // Initialize the database
        insertedCloudService = cloudServiceRepository.saveAndFlush(cloudService);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the cloudService using partial update
        CloudService partialUpdatedCloudService = new CloudService();
        partialUpdatedCloudService.setId(cloudService.getId());

        partialUpdatedCloudService.acronym(UPDATED_ACRONYM).name(UPDATED_NAME).type(UPDATED_TYPE).description(UPDATED_DESCRIPTION);

        restCloudServiceMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedCloudService.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedCloudService))
            )
            .andExpect(status().isOk());

        // Validate the CloudService in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertCloudServiceUpdatableFieldsEquals(partialUpdatedCloudService, getPersistedCloudService(partialUpdatedCloudService));
    }

    @Test
    @Transactional
    void patchNonExistingCloudService() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudService.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restCloudServiceMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, cloudService.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(cloudService))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchCloudService() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudService.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(cloudService))
            )
            .andExpect(status().isBadRequest());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamCloudService() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        cloudService.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restCloudServiceMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(cloudService))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the CloudService in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteCloudService() throws Exception {
        // Initialize the database
        insertedCloudService = cloudServiceRepository.saveAndFlush(cloudService);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the cloudService
        restCloudServiceMockMvc
            .perform(delete(ENTITY_API_URL_ID, cloudService.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return cloudServiceRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected CloudService getPersistedCloudService(CloudService cloudService) {
        return cloudServiceRepository.findById(cloudService.getId()).orElseThrow();
    }

    protected void assertPersistedCloudServiceToMatchAllProperties(CloudService expectedCloudService) {
        assertCloudServiceAllPropertiesEquals(expectedCloudService, getPersistedCloudService(expectedCloudService));
    }

    protected void assertPersistedCloudServiceToMatchUpdatableProperties(CloudService expectedCloudService) {
        assertCloudServiceAllUpdatablePropertiesEquals(expectedCloudService, getPersistedCloudService(expectedCloudService));
    }
}
