package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.QuestionnaireNonConformityAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.QuestionnaireNonConformity;
import com.emerald.rcm.backend.repository.QuestionnaireNonConformityRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link QuestionnaireNonConformityResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class QuestionnaireNonConformityResourceIT {

    private static final String DEFAULT_QUESTIONNAIRE_NAME = "AAAAAAAAAA";
    private static final String UPDATED_QUESTIONNAIRE_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_COMMENTS = "AAAAAAAAAA";
    private static final String UPDATED_COMMENTS = "BBBBBBBBBB";

    private static final String DEFAULT_COMPLIANCE = "AAAAAAAAAA";
    private static final String UPDATED_COMPLIANCE = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/questionnaire-non-conformities";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private QuestionnaireNonConformityRepository questionnaireNonConformityRepository;

    @Mock
    private QuestionnaireNonConformityRepository questionnaireNonConformityRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restQuestionnaireNonConformityMockMvc;

    private QuestionnaireNonConformity questionnaireNonConformity;

    private QuestionnaireNonConformity insertedQuestionnaireNonConformity;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static QuestionnaireNonConformity createEntity(EntityManager em) {
        QuestionnaireNonConformity questionnaireNonConformity = new QuestionnaireNonConformity()
            .questionnaireName(DEFAULT_QUESTIONNAIRE_NAME)
            .comments(DEFAULT_COMMENTS)
            .compliance(DEFAULT_COMPLIANCE);
        return questionnaireNonConformity;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static QuestionnaireNonConformity createUpdatedEntity(EntityManager em) {
        QuestionnaireNonConformity questionnaireNonConformity = new QuestionnaireNonConformity()
            .questionnaireName(UPDATED_QUESTIONNAIRE_NAME)
            .comments(UPDATED_COMMENTS)
            .compliance(UPDATED_COMPLIANCE);
        return questionnaireNonConformity;
    }

    @BeforeEach
    public void initTest() {
        questionnaireNonConformity = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedQuestionnaireNonConformity != null) {
            questionnaireNonConformityRepository.delete(insertedQuestionnaireNonConformity);
            insertedQuestionnaireNonConformity = null;
        }
    }

    @Test
    @Transactional
    void createQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the QuestionnaireNonConformity
        var returnedQuestionnaireNonConformity = om.readValue(
            restQuestionnaireNonConformityMockMvc
                .perform(
                    post(ENTITY_API_URL)
                        .with(csrf())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(om.writeValueAsBytes(questionnaireNonConformity))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            QuestionnaireNonConformity.class
        );

        // Validate the QuestionnaireNonConformity in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertQuestionnaireNonConformityUpdatableFieldsEquals(
            returnedQuestionnaireNonConformity,
            getPersistedQuestionnaireNonConformity(returnedQuestionnaireNonConformity)
        );

        insertedQuestionnaireNonConformity = returnedQuestionnaireNonConformity;
    }

    @Test
    @Transactional
    void createQuestionnaireNonConformityWithExistingId() throws Exception {
        // Create the QuestionnaireNonConformity with an existing ID
        questionnaireNonConformity.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restQuestionnaireNonConformityMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkQuestionnaireNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionnaireNonConformity.setQuestionnaireName(null);

        // Create the QuestionnaireNonConformity, which fails.

        restQuestionnaireNonConformityMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkComplianceIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionnaireNonConformity.setCompliance(null);

        // Create the QuestionnaireNonConformity, which fails.

        restQuestionnaireNonConformityMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllQuestionnaireNonConformities() throws Exception {
        // Initialize the database
        insertedQuestionnaireNonConformity = questionnaireNonConformityRepository.saveAndFlush(questionnaireNonConformity);

        // Get all the questionnaireNonConformityList
        restQuestionnaireNonConformityMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(questionnaireNonConformity.getId().intValue())))
            .andExpect(jsonPath("$.[*].questionnaireName").value(hasItem(DEFAULT_QUESTIONNAIRE_NAME)))
            .andExpect(jsonPath("$.[*].comments").value(hasItem(DEFAULT_COMMENTS.toString())))
            .andExpect(jsonPath("$.[*].compliance").value(hasItem(DEFAULT_COMPLIANCE)));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllQuestionnaireNonConformitiesWithEagerRelationshipsIsEnabled() throws Exception {
        when(questionnaireNonConformityRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restQuestionnaireNonConformityMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(questionnaireNonConformityRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllQuestionnaireNonConformitiesWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(questionnaireNonConformityRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restQuestionnaireNonConformityMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(questionnaireNonConformityRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getQuestionnaireNonConformity() throws Exception {
        // Initialize the database
        insertedQuestionnaireNonConformity = questionnaireNonConformityRepository.saveAndFlush(questionnaireNonConformity);

        // Get the questionnaireNonConformity
        restQuestionnaireNonConformityMockMvc
            .perform(get(ENTITY_API_URL_ID, questionnaireNonConformity.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(questionnaireNonConformity.getId().intValue()))
            .andExpect(jsonPath("$.questionnaireName").value(DEFAULT_QUESTIONNAIRE_NAME))
            .andExpect(jsonPath("$.comments").value(DEFAULT_COMMENTS.toString()))
            .andExpect(jsonPath("$.compliance").value(DEFAULT_COMPLIANCE));
    }

    @Test
    @Transactional
    void getNonExistingQuestionnaireNonConformity() throws Exception {
        // Get the questionnaireNonConformity
        restQuestionnaireNonConformityMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingQuestionnaireNonConformity() throws Exception {
        // Initialize the database
        insertedQuestionnaireNonConformity = questionnaireNonConformityRepository.saveAndFlush(questionnaireNonConformity);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionnaireNonConformity
        QuestionnaireNonConformity updatedQuestionnaireNonConformity = questionnaireNonConformityRepository
            .findById(questionnaireNonConformity.getId())
            .orElseThrow();
        // Disconnect from session so that the updates on updatedQuestionnaireNonConformity are not directly saved in db
        em.detach(updatedQuestionnaireNonConformity);
        updatedQuestionnaireNonConformity
            .questionnaireName(UPDATED_QUESTIONNAIRE_NAME)
            .comments(UPDATED_COMMENTS)
            .compliance(UPDATED_COMPLIANCE);

        restQuestionnaireNonConformityMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedQuestionnaireNonConformity.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedQuestionnaireNonConformity))
            )
            .andExpect(status().isOk());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedQuestionnaireNonConformityToMatchAllProperties(updatedQuestionnaireNonConformity);
    }

    @Test
    @Transactional
    void putNonExistingQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaireNonConformity.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionnaireNonConformityMockMvc
            .perform(
                put(ENTITY_API_URL_ID, questionnaireNonConformity.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaireNonConformity.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireNonConformityMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaireNonConformity.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireNonConformityMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateQuestionnaireNonConformityWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionnaireNonConformity = questionnaireNonConformityRepository.saveAndFlush(questionnaireNonConformity);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionnaireNonConformity using partial update
        QuestionnaireNonConformity partialUpdatedQuestionnaireNonConformity = new QuestionnaireNonConformity();
        partialUpdatedQuestionnaireNonConformity.setId(questionnaireNonConformity.getId());

        partialUpdatedQuestionnaireNonConformity.comments(UPDATED_COMMENTS).compliance(UPDATED_COMPLIANCE);

        restQuestionnaireNonConformityMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionnaireNonConformity.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionnaireNonConformity))
            )
            .andExpect(status().isOk());

        // Validate the QuestionnaireNonConformity in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionnaireNonConformityUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedQuestionnaireNonConformity, questionnaireNonConformity),
            getPersistedQuestionnaireNonConformity(questionnaireNonConformity)
        );
    }

    @Test
    @Transactional
    void fullUpdateQuestionnaireNonConformityWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionnaireNonConformity = questionnaireNonConformityRepository.saveAndFlush(questionnaireNonConformity);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionnaireNonConformity using partial update
        QuestionnaireNonConformity partialUpdatedQuestionnaireNonConformity = new QuestionnaireNonConformity();
        partialUpdatedQuestionnaireNonConformity.setId(questionnaireNonConformity.getId());

        partialUpdatedQuestionnaireNonConformity
            .questionnaireName(UPDATED_QUESTIONNAIRE_NAME)
            .comments(UPDATED_COMMENTS)
            .compliance(UPDATED_COMPLIANCE);

        restQuestionnaireNonConformityMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionnaireNonConformity.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionnaireNonConformity))
            )
            .andExpect(status().isOk());

        // Validate the QuestionnaireNonConformity in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionnaireNonConformityUpdatableFieldsEquals(
            partialUpdatedQuestionnaireNonConformity,
            getPersistedQuestionnaireNonConformity(partialUpdatedQuestionnaireNonConformity)
        );
    }

    @Test
    @Transactional
    void patchNonExistingQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaireNonConformity.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionnaireNonConformityMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, questionnaireNonConformity.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaireNonConformity.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireNonConformityMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamQuestionnaireNonConformity() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaireNonConformity.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireNonConformityMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionnaireNonConformity))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the QuestionnaireNonConformity in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteQuestionnaireNonConformity() throws Exception {
        // Initialize the database
        insertedQuestionnaireNonConformity = questionnaireNonConformityRepository.saveAndFlush(questionnaireNonConformity);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the questionnaireNonConformity
        restQuestionnaireNonConformityMockMvc
            .perform(delete(ENTITY_API_URL_ID, questionnaireNonConformity.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return questionnaireNonConformityRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected QuestionnaireNonConformity getPersistedQuestionnaireNonConformity(QuestionnaireNonConformity questionnaireNonConformity) {
        return questionnaireNonConformityRepository.findById(questionnaireNonConformity.getId()).orElseThrow();
    }

    protected void assertPersistedQuestionnaireNonConformityToMatchAllProperties(
        QuestionnaireNonConformity expectedQuestionnaireNonConformity
    ) {
        assertQuestionnaireNonConformityAllPropertiesEquals(
            expectedQuestionnaireNonConformity,
            getPersistedQuestionnaireNonConformity(expectedQuestionnaireNonConformity)
        );
    }

    protected void assertPersistedQuestionnaireNonConformityToMatchUpdatableProperties(
        QuestionnaireNonConformity expectedQuestionnaireNonConformity
    ) {
        assertQuestionnaireNonConformityAllUpdatablePropertiesEquals(
            expectedQuestionnaireNonConformity,
            getPersistedQuestionnaireNonConformity(expectedQuestionnaireNonConformity)
        );
    }
}
