package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.QuestionnaireAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.QuestionAnswer;
import com.emerald.rcm.backend.domain.Questionnaire;
import com.emerald.rcm.backend.domain.User;
import com.emerald.rcm.backend.repository.QuestionnaireRepository;
import com.emerald.rcm.backend.repository.UserRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link QuestionnaireResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class QuestionnaireResourceIT {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_EVIDENCES = "AAAAAAAAAA";
    private static final String UPDATED_EVIDENCES = "BBBBBBBBBB";

    private static final String DEFAULT_COMMENTS = "AAAAAAAAAA";
    private static final String UPDATED_COMMENTS = "BBBBBBBBBB";

    private static final Instant DEFAULT_LAST_UPDATE = Instant.ofEpochMilli(0L);
    private static final Instant UPDATED_LAST_UPDATE = Instant.now().truncatedTo(ChronoUnit.MILLIS);

    private static final String ENTITY_API_URL = "/api/questionnaires";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private QuestionnaireRepository questionnaireRepository;

    @Autowired
    private UserRepository userRepository;

    @Mock
    private QuestionnaireRepository questionnaireRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restQuestionnaireMockMvc;

    private Questionnaire questionnaire;

    private Questionnaire insertedQuestionnaire;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Questionnaire createEntity(EntityManager em) {
        Questionnaire questionnaire = new Questionnaire()
            .name(DEFAULT_NAME)
            .evidences(DEFAULT_EVIDENCES)
            .comments(DEFAULT_COMMENTS)
            .lastUpdate(DEFAULT_LAST_UPDATE);
        // Add required entity
        QuestionAnswer questionAnswer;
        if (TestUtil.findAll(em, QuestionAnswer.class).isEmpty()) {
            questionAnswer = QuestionAnswerResourceIT.createEntity(em);
            em.persist(questionAnswer);
            em.flush();
        } else {
            questionAnswer = TestUtil.findAll(em, QuestionAnswer.class).get(0);
        }
        questionnaire.setQuestionAnswer(questionAnswer);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        questionnaire.setUser(user);
        return questionnaire;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Questionnaire createUpdatedEntity(EntityManager em) {
        Questionnaire questionnaire = new Questionnaire()
            .name(UPDATED_NAME)
            .evidences(UPDATED_EVIDENCES)
            .comments(UPDATED_COMMENTS)
            .lastUpdate(UPDATED_LAST_UPDATE);
        // Add required entity
        QuestionAnswer questionAnswer;
        if (TestUtil.findAll(em, QuestionAnswer.class).isEmpty()) {
            questionAnswer = QuestionAnswerResourceIT.createUpdatedEntity(em);
            em.persist(questionAnswer);
            em.flush();
        } else {
            questionAnswer = TestUtil.findAll(em, QuestionAnswer.class).get(0);
        }
        questionnaire.setQuestionAnswer(questionAnswer);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        questionnaire.setUser(user);
        return questionnaire;
    }

    @BeforeEach
    public void initTest() {
        questionnaire = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedQuestionnaire != null) {
            questionnaireRepository.delete(insertedQuestionnaire);
            insertedQuestionnaire = null;
        }
        userRepository.deleteAll();
    }

    @Test
    @Transactional
    void createQuestionnaire() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the Questionnaire
        var returnedQuestionnaire = om.readValue(
            restQuestionnaireMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionnaire))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            Questionnaire.class
        );

        // Validate the Questionnaire in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertQuestionnaireUpdatableFieldsEquals(returnedQuestionnaire, getPersistedQuestionnaire(returnedQuestionnaire));

        insertedQuestionnaire = returnedQuestionnaire;
    }

    @Test
    @Transactional
    void createQuestionnaireWithExistingId() throws Exception {
        // Create the Questionnaire with an existing ID
        questionnaire.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restQuestionnaireMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionnaire)))
            .andExpect(status().isBadRequest());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionnaire.setName(null);

        // Create the Questionnaire, which fails.

        restQuestionnaireMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionnaire)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkLastUpdateIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionnaire.setLastUpdate(null);

        // Create the Questionnaire, which fails.

        restQuestionnaireMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionnaire)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllQuestionnaires() throws Exception {
        // Initialize the database
        insertedQuestionnaire = questionnaireRepository.saveAndFlush(questionnaire);

        // Get all the questionnaireList
        restQuestionnaireMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(questionnaire.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].evidences").value(hasItem(DEFAULT_EVIDENCES.toString())))
            .andExpect(jsonPath("$.[*].comments").value(hasItem(DEFAULT_COMMENTS.toString())))
            .andExpect(jsonPath("$.[*].lastUpdate").value(hasItem(DEFAULT_LAST_UPDATE.toString())));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllQuestionnairesWithEagerRelationshipsIsEnabled() throws Exception {
        when(questionnaireRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restQuestionnaireMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(questionnaireRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllQuestionnairesWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(questionnaireRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restQuestionnaireMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(questionnaireRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getQuestionnaire() throws Exception {
        // Initialize the database
        insertedQuestionnaire = questionnaireRepository.saveAndFlush(questionnaire);

        // Get the questionnaire
        restQuestionnaireMockMvc
            .perform(get(ENTITY_API_URL_ID, questionnaire.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(questionnaire.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.evidences").value(DEFAULT_EVIDENCES.toString()))
            .andExpect(jsonPath("$.comments").value(DEFAULT_COMMENTS.toString()))
            .andExpect(jsonPath("$.lastUpdate").value(DEFAULT_LAST_UPDATE.toString()));
    }

    @Test
    @Transactional
    void getNonExistingQuestionnaire() throws Exception {
        // Get the questionnaire
        restQuestionnaireMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingQuestionnaire() throws Exception {
        // Initialize the database
        insertedQuestionnaire = questionnaireRepository.saveAndFlush(questionnaire);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionnaire
        Questionnaire updatedQuestionnaire = questionnaireRepository.findById(questionnaire.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedQuestionnaire are not directly saved in db
        em.detach(updatedQuestionnaire);
        updatedQuestionnaire.name(UPDATED_NAME).evidences(UPDATED_EVIDENCES).comments(UPDATED_COMMENTS).lastUpdate(UPDATED_LAST_UPDATE);

        restQuestionnaireMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedQuestionnaire.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedQuestionnaire))
            )
            .andExpect(status().isOk());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedQuestionnaireToMatchAllProperties(updatedQuestionnaire);
    }

    @Test
    @Transactional
    void putNonExistingQuestionnaire() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaire.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionnaireMockMvc
            .perform(
                put(ENTITY_API_URL_ID, questionnaire.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaire))
            )
            .andExpect(status().isBadRequest());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchQuestionnaire() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaire.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionnaire))
            )
            .andExpect(status().isBadRequest());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamQuestionnaire() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaire.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionnaire)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateQuestionnaireWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionnaire = questionnaireRepository.saveAndFlush(questionnaire);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionnaire using partial update
        Questionnaire partialUpdatedQuestionnaire = new Questionnaire();
        partialUpdatedQuestionnaire.setId(questionnaire.getId());

        partialUpdatedQuestionnaire.evidences(UPDATED_EVIDENCES).comments(UPDATED_COMMENTS);

        restQuestionnaireMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionnaire.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionnaire))
            )
            .andExpect(status().isOk());

        // Validate the Questionnaire in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionnaireUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedQuestionnaire, questionnaire),
            getPersistedQuestionnaire(questionnaire)
        );
    }

    @Test
    @Transactional
    void fullUpdateQuestionnaireWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionnaire = questionnaireRepository.saveAndFlush(questionnaire);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionnaire using partial update
        Questionnaire partialUpdatedQuestionnaire = new Questionnaire();
        partialUpdatedQuestionnaire.setId(questionnaire.getId());

        partialUpdatedQuestionnaire
            .name(UPDATED_NAME)
            .evidences(UPDATED_EVIDENCES)
            .comments(UPDATED_COMMENTS)
            .lastUpdate(UPDATED_LAST_UPDATE);

        restQuestionnaireMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionnaire.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionnaire))
            )
            .andExpect(status().isOk());

        // Validate the Questionnaire in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionnaireUpdatableFieldsEquals(partialUpdatedQuestionnaire, getPersistedQuestionnaire(partialUpdatedQuestionnaire));
    }

    @Test
    @Transactional
    void patchNonExistingQuestionnaire() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaire.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionnaireMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, questionnaire.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionnaire))
            )
            .andExpect(status().isBadRequest());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchQuestionnaire() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaire.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionnaire))
            )
            .andExpect(status().isBadRequest());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamQuestionnaire() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionnaire.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionnaireMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(questionnaire))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the Questionnaire in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteQuestionnaire() throws Exception {
        // Initialize the database
        insertedQuestionnaire = questionnaireRepository.saveAndFlush(questionnaire);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the questionnaire
        restQuestionnaireMockMvc
            .perform(delete(ENTITY_API_URL_ID, questionnaire.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return questionnaireRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected Questionnaire getPersistedQuestionnaire(Questionnaire questionnaire) {
        return questionnaireRepository.findById(questionnaire.getId()).orElseThrow();
    }

    protected void assertPersistedQuestionnaireToMatchAllProperties(Questionnaire expectedQuestionnaire) {
        assertQuestionnaireAllPropertiesEquals(expectedQuestionnaire, getPersistedQuestionnaire(expectedQuestionnaire));
    }

    protected void assertPersistedQuestionnaireToMatchUpdatableProperties(Questionnaire expectedQuestionnaire) {
        assertQuestionnaireAllUpdatablePropertiesEquals(expectedQuestionnaire, getPersistedQuestionnaire(expectedQuestionnaire));
    }
}
