package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.ReferenceTomAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.ReferenceTom;
import com.emerald.rcm.backend.repository.ReferenceTomRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link ReferenceTomResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class ReferenceTomResourceIT {

    private static final String DEFAULT_REFERENCE_TOM_IMPL = "AAAAAAAAAA";
    private static final String UPDATED_REFERENCE_TOM_IMPL = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/reference-toms";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private ReferenceTomRepository referenceTomRepository;

    @Mock
    private ReferenceTomRepository referenceTomRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restReferenceTomMockMvc;

    private ReferenceTom referenceTom;

    private ReferenceTom insertedReferenceTom;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ReferenceTom createEntity(EntityManager em) {
        ReferenceTom referenceTom = new ReferenceTom().referenceTomImpl(DEFAULT_REFERENCE_TOM_IMPL);
        return referenceTom;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ReferenceTom createUpdatedEntity(EntityManager em) {
        ReferenceTom referenceTom = new ReferenceTom().referenceTomImpl(UPDATED_REFERENCE_TOM_IMPL);
        return referenceTom;
    }

    @BeforeEach
    public void initTest() {
        referenceTom = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedReferenceTom != null) {
            referenceTomRepository.delete(insertedReferenceTom);
            insertedReferenceTom = null;
        }
    }

    @Test
    @Transactional
    void createReferenceTom() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the ReferenceTom
        var returnedReferenceTom = om.readValue(
            restReferenceTomMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(referenceTom))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            ReferenceTom.class
        );

        // Validate the ReferenceTom in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertReferenceTomUpdatableFieldsEquals(returnedReferenceTom, getPersistedReferenceTom(returnedReferenceTom));

        insertedReferenceTom = returnedReferenceTom;
    }

    @Test
    @Transactional
    void createReferenceTomWithExistingId() throws Exception {
        // Create the ReferenceTom with an existing ID
        referenceTom.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restReferenceTomMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(referenceTom)))
            .andExpect(status().isBadRequest());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkReferenceTomImplIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        referenceTom.setReferenceTomImpl(null);

        // Create the ReferenceTom, which fails.

        restReferenceTomMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(referenceTom)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllReferenceToms() throws Exception {
        // Initialize the database
        insertedReferenceTom = referenceTomRepository.saveAndFlush(referenceTom);

        // Get all the referenceTomList
        restReferenceTomMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(referenceTom.getId().intValue())))
            .andExpect(jsonPath("$.[*].referenceTomImpl").value(hasItem(DEFAULT_REFERENCE_TOM_IMPL)));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllReferenceTomsWithEagerRelationshipsIsEnabled() throws Exception {
        when(referenceTomRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restReferenceTomMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(referenceTomRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllReferenceTomsWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(referenceTomRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restReferenceTomMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(referenceTomRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getReferenceTom() throws Exception {
        // Initialize the database
        insertedReferenceTom = referenceTomRepository.saveAndFlush(referenceTom);

        // Get the referenceTom
        restReferenceTomMockMvc
            .perform(get(ENTITY_API_URL_ID, referenceTom.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(referenceTom.getId().intValue()))
            .andExpect(jsonPath("$.referenceTomImpl").value(DEFAULT_REFERENCE_TOM_IMPL));
    }

    @Test
    @Transactional
    void getNonExistingReferenceTom() throws Exception {
        // Get the referenceTom
        restReferenceTomMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingReferenceTom() throws Exception {
        // Initialize the database
        insertedReferenceTom = referenceTomRepository.saveAndFlush(referenceTom);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the referenceTom
        ReferenceTom updatedReferenceTom = referenceTomRepository.findById(referenceTom.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedReferenceTom are not directly saved in db
        em.detach(updatedReferenceTom);
        updatedReferenceTom.referenceTomImpl(UPDATED_REFERENCE_TOM_IMPL);

        restReferenceTomMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedReferenceTom.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedReferenceTom))
            )
            .andExpect(status().isOk());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedReferenceTomToMatchAllProperties(updatedReferenceTom);
    }

    @Test
    @Transactional
    void putNonExistingReferenceTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        referenceTom.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restReferenceTomMockMvc
            .perform(
                put(ENTITY_API_URL_ID, referenceTom.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(referenceTom))
            )
            .andExpect(status().isBadRequest());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchReferenceTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        referenceTom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReferenceTomMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(referenceTom))
            )
            .andExpect(status().isBadRequest());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamReferenceTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        referenceTom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReferenceTomMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(referenceTom)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateReferenceTomWithPatch() throws Exception {
        // Initialize the database
        insertedReferenceTom = referenceTomRepository.saveAndFlush(referenceTom);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the referenceTom using partial update
        ReferenceTom partialUpdatedReferenceTom = new ReferenceTom();
        partialUpdatedReferenceTom.setId(referenceTom.getId());

        restReferenceTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedReferenceTom.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedReferenceTom))
            )
            .andExpect(status().isOk());

        // Validate the ReferenceTom in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertReferenceTomUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedReferenceTom, referenceTom),
            getPersistedReferenceTom(referenceTom)
        );
    }

    @Test
    @Transactional
    void fullUpdateReferenceTomWithPatch() throws Exception {
        // Initialize the database
        insertedReferenceTom = referenceTomRepository.saveAndFlush(referenceTom);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the referenceTom using partial update
        ReferenceTom partialUpdatedReferenceTom = new ReferenceTom();
        partialUpdatedReferenceTom.setId(referenceTom.getId());

        partialUpdatedReferenceTom.referenceTomImpl(UPDATED_REFERENCE_TOM_IMPL);

        restReferenceTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedReferenceTom.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedReferenceTom))
            )
            .andExpect(status().isOk());

        // Validate the ReferenceTom in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertReferenceTomUpdatableFieldsEquals(partialUpdatedReferenceTom, getPersistedReferenceTom(partialUpdatedReferenceTom));
    }

    @Test
    @Transactional
    void patchNonExistingReferenceTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        referenceTom.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restReferenceTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, referenceTom.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(referenceTom))
            )
            .andExpect(status().isBadRequest());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchReferenceTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        referenceTom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReferenceTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(referenceTom))
            )
            .andExpect(status().isBadRequest());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamReferenceTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        referenceTom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReferenceTomMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(referenceTom))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the ReferenceTom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteReferenceTom() throws Exception {
        // Initialize the database
        insertedReferenceTom = referenceTomRepository.saveAndFlush(referenceTom);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the referenceTom
        restReferenceTomMockMvc
            .perform(delete(ENTITY_API_URL_ID, referenceTom.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return referenceTomRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected ReferenceTom getPersistedReferenceTom(ReferenceTom referenceTom) {
        return referenceTomRepository.findById(referenceTom.getId()).orElseThrow();
    }

    protected void assertPersistedReferenceTomToMatchAllProperties(ReferenceTom expectedReferenceTom) {
        assertReferenceTomAllPropertiesEquals(expectedReferenceTom, getPersistedReferenceTom(expectedReferenceTom));
    }

    protected void assertPersistedReferenceTomToMatchUpdatableProperties(ReferenceTom expectedReferenceTom) {
        assertReferenceTomAllUpdatablePropertiesEquals(expectedReferenceTom, getPersistedReferenceTom(expectedReferenceTom));
    }
}
