package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.SecurityControlCategoryAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.SecurityControlCategory;
import com.emerald.rcm.backend.repository.SecurityControlCategoryRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link SecurityControlCategoryResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class SecurityControlCategoryResourceIT {

    private static final String DEFAULT_CODE = "AAAAAAAAAA";
    private static final String UPDATED_CODE = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/security-control-categories";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private SecurityControlCategoryRepository securityControlCategoryRepository;

    @Mock
    private SecurityControlCategoryRepository securityControlCategoryRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSecurityControlCategoryMockMvc;

    private SecurityControlCategory securityControlCategory;

    private SecurityControlCategory insertedSecurityControlCategory;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityControlCategory createEntity(EntityManager em) {
        SecurityControlCategory securityControlCategory = new SecurityControlCategory()
            .code(DEFAULT_CODE)
            .name(DEFAULT_NAME)
            .description(DEFAULT_DESCRIPTION);
        return securityControlCategory;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityControlCategory createUpdatedEntity(EntityManager em) {
        SecurityControlCategory securityControlCategory = new SecurityControlCategory()
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .description(UPDATED_DESCRIPTION);
        return securityControlCategory;
    }

    @BeforeEach
    public void initTest() {
        securityControlCategory = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedSecurityControlCategory != null) {
            securityControlCategoryRepository.delete(insertedSecurityControlCategory);
            insertedSecurityControlCategory = null;
        }
    }

    @Test
    @Transactional
    void createSecurityControlCategory() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the SecurityControlCategory
        var returnedSecurityControlCategory = om.readValue(
            restSecurityControlCategoryMockMvc
                .perform(
                    post(ENTITY_API_URL)
                        .with(csrf())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(om.writeValueAsBytes(securityControlCategory))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            SecurityControlCategory.class
        );

        // Validate the SecurityControlCategory in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertSecurityControlCategoryUpdatableFieldsEquals(
            returnedSecurityControlCategory,
            getPersistedSecurityControlCategory(returnedSecurityControlCategory)
        );

        insertedSecurityControlCategory = returnedSecurityControlCategory;
    }

    @Test
    @Transactional
    void createSecurityControlCategoryWithExistingId() throws Exception {
        // Create the SecurityControlCategory with an existing ID
        securityControlCategory.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSecurityControlCategoryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkCodeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControlCategory.setCode(null);

        // Create the SecurityControlCategory, which fails.

        restSecurityControlCategoryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControlCategory.setName(null);

        // Create the SecurityControlCategory, which fails.

        restSecurityControlCategoryMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSecurityControlCategories() throws Exception {
        // Initialize the database
        insertedSecurityControlCategory = securityControlCategoryRepository.saveAndFlush(securityControlCategory);

        // Get all the securityControlCategoryList
        restSecurityControlCategoryMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(securityControlCategory.getId().intValue())))
            .andExpect(jsonPath("$.[*].code").value(hasItem(DEFAULT_CODE)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION.toString())));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSecurityControlCategoriesWithEagerRelationshipsIsEnabled() throws Exception {
        when(securityControlCategoryRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSecurityControlCategoryMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(securityControlCategoryRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSecurityControlCategoriesWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(securityControlCategoryRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSecurityControlCategoryMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(securityControlCategoryRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getSecurityControlCategory() throws Exception {
        // Initialize the database
        insertedSecurityControlCategory = securityControlCategoryRepository.saveAndFlush(securityControlCategory);

        // Get the securityControlCategory
        restSecurityControlCategoryMockMvc
            .perform(get(ENTITY_API_URL_ID, securityControlCategory.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(securityControlCategory.getId().intValue()))
            .andExpect(jsonPath("$.code").value(DEFAULT_CODE))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION.toString()));
    }

    @Test
    @Transactional
    void getNonExistingSecurityControlCategory() throws Exception {
        // Get the securityControlCategory
        restSecurityControlCategoryMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingSecurityControlCategory() throws Exception {
        // Initialize the database
        insertedSecurityControlCategory = securityControlCategoryRepository.saveAndFlush(securityControlCategory);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControlCategory
        SecurityControlCategory updatedSecurityControlCategory = securityControlCategoryRepository
            .findById(securityControlCategory.getId())
            .orElseThrow();
        // Disconnect from session so that the updates on updatedSecurityControlCategory are not directly saved in db
        em.detach(updatedSecurityControlCategory);
        updatedSecurityControlCategory.code(UPDATED_CODE).name(UPDATED_NAME).description(UPDATED_DESCRIPTION);

        restSecurityControlCategoryMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedSecurityControlCategory.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedSecurityControlCategory))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedSecurityControlCategoryToMatchAllProperties(updatedSecurityControlCategory);
    }

    @Test
    @Transactional
    void putNonExistingSecurityControlCategory() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlCategory.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityControlCategoryMockMvc
            .perform(
                put(ENTITY_API_URL_ID, securityControlCategory.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSecurityControlCategory() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlCategory.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlCategoryMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSecurityControlCategory() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlCategory.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlCategoryMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSecurityControlCategoryWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityControlCategory = securityControlCategoryRepository.saveAndFlush(securityControlCategory);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControlCategory using partial update
        SecurityControlCategory partialUpdatedSecurityControlCategory = new SecurityControlCategory();
        partialUpdatedSecurityControlCategory.setId(securityControlCategory.getId());

        partialUpdatedSecurityControlCategory.code(UPDATED_CODE).description(UPDATED_DESCRIPTION);

        restSecurityControlCategoryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityControlCategory.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityControlCategory))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControlCategory in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityControlCategoryUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedSecurityControlCategory, securityControlCategory),
            getPersistedSecurityControlCategory(securityControlCategory)
        );
    }

    @Test
    @Transactional
    void fullUpdateSecurityControlCategoryWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityControlCategory = securityControlCategoryRepository.saveAndFlush(securityControlCategory);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControlCategory using partial update
        SecurityControlCategory partialUpdatedSecurityControlCategory = new SecurityControlCategory();
        partialUpdatedSecurityControlCategory.setId(securityControlCategory.getId());

        partialUpdatedSecurityControlCategory.code(UPDATED_CODE).name(UPDATED_NAME).description(UPDATED_DESCRIPTION);

        restSecurityControlCategoryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityControlCategory.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityControlCategory))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControlCategory in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityControlCategoryUpdatableFieldsEquals(
            partialUpdatedSecurityControlCategory,
            getPersistedSecurityControlCategory(partialUpdatedSecurityControlCategory)
        );
    }

    @Test
    @Transactional
    void patchNonExistingSecurityControlCategory() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlCategory.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityControlCategoryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, securityControlCategory.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSecurityControlCategory() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlCategory.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlCategoryMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSecurityControlCategory() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlCategory.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlCategoryMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControlCategory))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityControlCategory in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSecurityControlCategory() throws Exception {
        // Initialize the database
        insertedSecurityControlCategory = securityControlCategoryRepository.saveAndFlush(securityControlCategory);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the securityControlCategory
        restSecurityControlCategoryMockMvc
            .perform(delete(ENTITY_API_URL_ID, securityControlCategory.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return securityControlCategoryRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected SecurityControlCategory getPersistedSecurityControlCategory(SecurityControlCategory securityControlCategory) {
        return securityControlCategoryRepository.findById(securityControlCategory.getId()).orElseThrow();
    }

    protected void assertPersistedSecurityControlCategoryToMatchAllProperties(SecurityControlCategory expectedSecurityControlCategory) {
        assertSecurityControlCategoryAllPropertiesEquals(
            expectedSecurityControlCategory,
            getPersistedSecurityControlCategory(expectedSecurityControlCategory)
        );
    }

    protected void assertPersistedSecurityControlCategoryToMatchUpdatableProperties(
        SecurityControlCategory expectedSecurityControlCategory
    ) {
        assertSecurityControlCategoryAllUpdatablePropertiesEquals(
            expectedSecurityControlCategory,
            getPersistedSecurityControlCategory(expectedSecurityControlCategory)
        );
    }
}
