package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.SecurityControlFrameworkAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.SecurityControlFramework;
import com.emerald.rcm.backend.repository.SecurityControlFrameworkRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link SecurityControlFrameworkResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@AutoConfigureMockMvc
@WithMockUser
class SecurityControlFrameworkResourceIT {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final String DEFAULT_VERSION = "AAAAAAAAAA";
    private static final String UPDATED_VERSION = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/security-control-frameworks";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private SecurityControlFrameworkRepository securityControlFrameworkRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSecurityControlFrameworkMockMvc;

    private SecurityControlFramework securityControlFramework;

    private SecurityControlFramework insertedSecurityControlFramework;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityControlFramework createEntity(EntityManager em) {
        SecurityControlFramework securityControlFramework = new SecurityControlFramework()
            .name(DEFAULT_NAME)
            .description(DEFAULT_DESCRIPTION)
            .version(DEFAULT_VERSION);
        return securityControlFramework;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityControlFramework createUpdatedEntity(EntityManager em) {
        SecurityControlFramework securityControlFramework = new SecurityControlFramework()
            .name(UPDATED_NAME)
            .description(UPDATED_DESCRIPTION)
            .version(UPDATED_VERSION);
        return securityControlFramework;
    }

    @BeforeEach
    public void initTest() {
        securityControlFramework = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedSecurityControlFramework != null) {
            securityControlFrameworkRepository.delete(insertedSecurityControlFramework);
            insertedSecurityControlFramework = null;
        }
    }

    @Test
    @Transactional
    void createSecurityControlFramework() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the SecurityControlFramework
        var returnedSecurityControlFramework = om.readValue(
            restSecurityControlFrameworkMockMvc
                .perform(
                    post(ENTITY_API_URL)
                        .with(csrf())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(om.writeValueAsBytes(securityControlFramework))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            SecurityControlFramework.class
        );

        // Validate the SecurityControlFramework in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertSecurityControlFrameworkUpdatableFieldsEquals(
            returnedSecurityControlFramework,
            getPersistedSecurityControlFramework(returnedSecurityControlFramework)
        );

        insertedSecurityControlFramework = returnedSecurityControlFramework;
    }

    @Test
    @Transactional
    void createSecurityControlFrameworkWithExistingId() throws Exception {
        // Create the SecurityControlFramework with an existing ID
        securityControlFramework.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSecurityControlFrameworkMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControlFramework.setName(null);

        // Create the SecurityControlFramework, which fails.

        restSecurityControlFrameworkMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkVersionIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControlFramework.setVersion(null);

        // Create the SecurityControlFramework, which fails.

        restSecurityControlFrameworkMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSecurityControlFrameworks() throws Exception {
        // Initialize the database
        insertedSecurityControlFramework = securityControlFrameworkRepository.saveAndFlush(securityControlFramework);

        // Get all the securityControlFrameworkList
        restSecurityControlFrameworkMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(securityControlFramework.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION.toString())))
            .andExpect(jsonPath("$.[*].version").value(hasItem(DEFAULT_VERSION)));
    }

    @Test
    @Transactional
    void getSecurityControlFramework() throws Exception {
        // Initialize the database
        insertedSecurityControlFramework = securityControlFrameworkRepository.saveAndFlush(securityControlFramework);

        // Get the securityControlFramework
        restSecurityControlFrameworkMockMvc
            .perform(get(ENTITY_API_URL_ID, securityControlFramework.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(securityControlFramework.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION.toString()))
            .andExpect(jsonPath("$.version").value(DEFAULT_VERSION));
    }

    @Test
    @Transactional
    void getNonExistingSecurityControlFramework() throws Exception {
        // Get the securityControlFramework
        restSecurityControlFrameworkMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingSecurityControlFramework() throws Exception {
        // Initialize the database
        insertedSecurityControlFramework = securityControlFrameworkRepository.saveAndFlush(securityControlFramework);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControlFramework
        SecurityControlFramework updatedSecurityControlFramework = securityControlFrameworkRepository
            .findById(securityControlFramework.getId())
            .orElseThrow();
        // Disconnect from session so that the updates on updatedSecurityControlFramework are not directly saved in db
        em.detach(updatedSecurityControlFramework);
        updatedSecurityControlFramework.name(UPDATED_NAME).description(UPDATED_DESCRIPTION).version(UPDATED_VERSION);

        restSecurityControlFrameworkMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedSecurityControlFramework.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedSecurityControlFramework))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedSecurityControlFrameworkToMatchAllProperties(updatedSecurityControlFramework);
    }

    @Test
    @Transactional
    void putNonExistingSecurityControlFramework() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlFramework.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityControlFrameworkMockMvc
            .perform(
                put(ENTITY_API_URL_ID, securityControlFramework.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSecurityControlFramework() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlFramework.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlFrameworkMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSecurityControlFramework() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlFramework.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlFrameworkMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSecurityControlFrameworkWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityControlFramework = securityControlFrameworkRepository.saveAndFlush(securityControlFramework);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControlFramework using partial update
        SecurityControlFramework partialUpdatedSecurityControlFramework = new SecurityControlFramework();
        partialUpdatedSecurityControlFramework.setId(securityControlFramework.getId());

        partialUpdatedSecurityControlFramework.name(UPDATED_NAME).version(UPDATED_VERSION);

        restSecurityControlFrameworkMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityControlFramework.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityControlFramework))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControlFramework in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityControlFrameworkUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedSecurityControlFramework, securityControlFramework),
            getPersistedSecurityControlFramework(securityControlFramework)
        );
    }

    @Test
    @Transactional
    void fullUpdateSecurityControlFrameworkWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityControlFramework = securityControlFrameworkRepository.saveAndFlush(securityControlFramework);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControlFramework using partial update
        SecurityControlFramework partialUpdatedSecurityControlFramework = new SecurityControlFramework();
        partialUpdatedSecurityControlFramework.setId(securityControlFramework.getId());

        partialUpdatedSecurityControlFramework.name(UPDATED_NAME).description(UPDATED_DESCRIPTION).version(UPDATED_VERSION);

        restSecurityControlFrameworkMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityControlFramework.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityControlFramework))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControlFramework in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityControlFrameworkUpdatableFieldsEquals(
            partialUpdatedSecurityControlFramework,
            getPersistedSecurityControlFramework(partialUpdatedSecurityControlFramework)
        );
    }

    @Test
    @Transactional
    void patchNonExistingSecurityControlFramework() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlFramework.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityControlFrameworkMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, securityControlFramework.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSecurityControlFramework() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlFramework.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlFrameworkMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSecurityControlFramework() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControlFramework.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlFrameworkMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControlFramework))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityControlFramework in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSecurityControlFramework() throws Exception {
        // Initialize the database
        insertedSecurityControlFramework = securityControlFrameworkRepository.saveAndFlush(securityControlFramework);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the securityControlFramework
        restSecurityControlFrameworkMockMvc
            .perform(delete(ENTITY_API_URL_ID, securityControlFramework.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return securityControlFrameworkRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected SecurityControlFramework getPersistedSecurityControlFramework(SecurityControlFramework securityControlFramework) {
        return securityControlFrameworkRepository.findById(securityControlFramework.getId()).orElseThrow();
    }

    protected void assertPersistedSecurityControlFrameworkToMatchAllProperties(SecurityControlFramework expectedSecurityControlFramework) {
        assertSecurityControlFrameworkAllPropertiesEquals(
            expectedSecurityControlFramework,
            getPersistedSecurityControlFramework(expectedSecurityControlFramework)
        );
    }

    protected void assertPersistedSecurityControlFrameworkToMatchUpdatableProperties(
        SecurityControlFramework expectedSecurityControlFramework
    ) {
        assertSecurityControlFrameworkAllUpdatablePropertiesEquals(
            expectedSecurityControlFramework,
            getPersistedSecurityControlFramework(expectedSecurityControlFramework)
        );
    }
}
