package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.SecurityControlAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.SecurityControl;
import com.emerald.rcm.backend.repository.SecurityControlRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link SecurityControlResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class SecurityControlResourceIT {

    private static final String DEFAULT_CODE = "AAAAAAAAAA";
    private static final String UPDATED_CODE = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_OBJECTIVE = "AAAAAAAAAA";
    private static final String UPDATED_OBJECTIVE = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/security-controls";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private SecurityControlRepository securityControlRepository;

    @Mock
    private SecurityControlRepository securityControlRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSecurityControlMockMvc;

    private SecurityControl securityControl;

    private SecurityControl insertedSecurityControl;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityControl createEntity(EntityManager em) {
        SecurityControl securityControl = new SecurityControl()
            .code(DEFAULT_CODE)
            .name(DEFAULT_NAME)
            .objective(DEFAULT_OBJECTIVE)
            .description(DEFAULT_DESCRIPTION);
        return securityControl;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityControl createUpdatedEntity(EntityManager em) {
        SecurityControl securityControl = new SecurityControl()
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .objective(UPDATED_OBJECTIVE)
            .description(UPDATED_DESCRIPTION);
        return securityControl;
    }

    @BeforeEach
    public void initTest() {
        securityControl = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedSecurityControl != null) {
            securityControlRepository.delete(insertedSecurityControl);
            insertedSecurityControl = null;
        }
    }

    @Test
    @Transactional
    void createSecurityControl() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the SecurityControl
        var returnedSecurityControl = om.readValue(
            restSecurityControlMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityControl))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            SecurityControl.class
        );

        // Validate the SecurityControl in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertSecurityControlUpdatableFieldsEquals(returnedSecurityControl, getPersistedSecurityControl(returnedSecurityControl));

        insertedSecurityControl = returnedSecurityControl;
    }

    @Test
    @Transactional
    void createSecurityControlWithExistingId() throws Exception {
        // Create the SecurityControl with an existing ID
        securityControl.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSecurityControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkCodeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControl.setCode(null);

        // Create the SecurityControl, which fails.

        restSecurityControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControl.setName(null);

        // Create the SecurityControl, which fails.

        restSecurityControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkObjectiveIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityControl.setObjective(null);

        // Create the SecurityControl, which fails.

        restSecurityControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSecurityControls() throws Exception {
        // Initialize the database
        insertedSecurityControl = securityControlRepository.saveAndFlush(securityControl);

        // Get all the securityControlList
        restSecurityControlMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(securityControl.getId().intValue())))
            .andExpect(jsonPath("$.[*].code").value(hasItem(DEFAULT_CODE)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].objective").value(hasItem(DEFAULT_OBJECTIVE)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION.toString())));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSecurityControlsWithEagerRelationshipsIsEnabled() throws Exception {
        when(securityControlRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSecurityControlMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(securityControlRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSecurityControlsWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(securityControlRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSecurityControlMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(securityControlRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getSecurityControl() throws Exception {
        // Initialize the database
        insertedSecurityControl = securityControlRepository.saveAndFlush(securityControl);

        // Get the securityControl
        restSecurityControlMockMvc
            .perform(get(ENTITY_API_URL_ID, securityControl.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(securityControl.getId().intValue()))
            .andExpect(jsonPath("$.code").value(DEFAULT_CODE))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.objective").value(DEFAULT_OBJECTIVE))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION.toString()));
    }

    @Test
    @Transactional
    void getNonExistingSecurityControl() throws Exception {
        // Get the securityControl
        restSecurityControlMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingSecurityControl() throws Exception {
        // Initialize the database
        insertedSecurityControl = securityControlRepository.saveAndFlush(securityControl);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControl
        SecurityControl updatedSecurityControl = securityControlRepository.findById(securityControl.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedSecurityControl are not directly saved in db
        em.detach(updatedSecurityControl);
        updatedSecurityControl.code(UPDATED_CODE).name(UPDATED_NAME).objective(UPDATED_OBJECTIVE).description(UPDATED_DESCRIPTION);

        restSecurityControlMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedSecurityControl.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedSecurityControl))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedSecurityControlToMatchAllProperties(updatedSecurityControl);
    }

    @Test
    @Transactional
    void putNonExistingSecurityControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControl.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityControlMockMvc
            .perform(
                put(ENTITY_API_URL_ID, securityControl.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSecurityControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSecurityControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlMockMvc
            .perform(
                put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSecurityControlWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityControl = securityControlRepository.saveAndFlush(securityControl);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControl using partial update
        SecurityControl partialUpdatedSecurityControl = new SecurityControl();
        partialUpdatedSecurityControl.setId(securityControl.getId());

        partialUpdatedSecurityControl.objective(UPDATED_OBJECTIVE).description(UPDATED_DESCRIPTION);

        restSecurityControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityControl.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityControl))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControl in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityControlUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedSecurityControl, securityControl),
            getPersistedSecurityControl(securityControl)
        );
    }

    @Test
    @Transactional
    void fullUpdateSecurityControlWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityControl = securityControlRepository.saveAndFlush(securityControl);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityControl using partial update
        SecurityControl partialUpdatedSecurityControl = new SecurityControl();
        partialUpdatedSecurityControl.setId(securityControl.getId());

        partialUpdatedSecurityControl.code(UPDATED_CODE).name(UPDATED_NAME).objective(UPDATED_OBJECTIVE).description(UPDATED_DESCRIPTION);

        restSecurityControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityControl.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityControl))
            )
            .andExpect(status().isOk());

        // Validate the SecurityControl in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityControlUpdatableFieldsEquals(
            partialUpdatedSecurityControl,
            getPersistedSecurityControl(partialUpdatedSecurityControl)
        );
    }

    @Test
    @Transactional
    void patchNonExistingSecurityControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControl.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, securityControl.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSecurityControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSecurityControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityControlMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityControl))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSecurityControl() throws Exception {
        // Initialize the database
        insertedSecurityControl = securityControlRepository.saveAndFlush(securityControl);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the securityControl
        restSecurityControlMockMvc
            .perform(delete(ENTITY_API_URL_ID, securityControl.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return securityControlRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected SecurityControl getPersistedSecurityControl(SecurityControl securityControl) {
        return securityControlRepository.findById(securityControl.getId()).orElseThrow();
    }

    protected void assertPersistedSecurityControlToMatchAllProperties(SecurityControl expectedSecurityControl) {
        assertSecurityControlAllPropertiesEquals(expectedSecurityControl, getPersistedSecurityControl(expectedSecurityControl));
    }

    protected void assertPersistedSecurityControlToMatchUpdatableProperties(SecurityControl expectedSecurityControl) {
        assertSecurityControlAllUpdatablePropertiesEquals(expectedSecurityControl, getPersistedSecurityControl(expectedSecurityControl));
    }
}
