package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.SecurityMetricAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.SecurityMetric;
import com.emerald.rcm.backend.repository.SecurityMetricRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link SecurityMetricResource} REST controller.
 */
@IntegrationTest
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class SecurityMetricResourceIT {

    private static final String DEFAULT_METRIC_ID = "AAAAAAAAAA";
    private static final String UPDATED_METRIC_ID = "BBBBBBBBBB";

    private static final String DEFAULT_CATEGORY = "AAAAAAAAAA";
    private static final String UPDATED_CATEGORY = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_SOURCE = "AAAAAAAAAA";
    private static final String UPDATED_SOURCE = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final String DEFAULT_SCALE = "AAAAAAAAAA";
    private static final String UPDATED_SCALE = "BBBBBBBBBB";

    private static final String DEFAULT_OPERATOR = "AAAAAAAAAA";
    private static final String UPDATED_OPERATOR = "BBBBBBBBBB";

    private static final String DEFAULT_TARGET_VALUE = "AAAAAAAAAA";
    private static final String UPDATED_TARGET_VALUE = "BBBBBBBBBB";

    private static final String DEFAULT_TARGET_VALUE_DATATYPE = "AAAAAAAAAA";
    private static final String UPDATED_TARGET_VALUE_DATATYPE = "BBBBBBBBBB";

    private static final String DEFAULT_INTERVAL = "AAAAAAAAAA";
    private static final String UPDATED_INTERVAL = "BBBBBBBBBB";

    private static final String DEFAULT_TARGET_RESOURCE_TYPE = "AAAAAAAAAA";
    private static final String UPDATED_TARGET_RESOURCE_TYPE = "BBBBBBBBBB";

    private static final String DEFAULT_SECURITY_FEATURE = "AAAAAAAAAA";
    private static final String UPDATED_SECURITY_FEATURE = "BBBBBBBBBB";

    private static final String DEFAULT_KEYWORDS = "AAAAAAAAAA";
    private static final String UPDATED_KEYWORDS = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/security-metrics";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private SecurityMetricRepository securityMetricRepository;

    @Mock
    private SecurityMetricRepository securityMetricRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSecurityMetricMockMvc;

    private SecurityMetric securityMetric;

    private SecurityMetric insertedSecurityMetric;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityMetric createEntity(EntityManager em) {
        SecurityMetric securityMetric = new SecurityMetric()
            .metricId(DEFAULT_METRIC_ID)
            .category(DEFAULT_CATEGORY)
            .name(DEFAULT_NAME)
            .source(DEFAULT_SOURCE)
            .description(DEFAULT_DESCRIPTION)
            .scale(DEFAULT_SCALE)
            .operator(DEFAULT_OPERATOR)
            .targetValue(DEFAULT_TARGET_VALUE)
            .targetValueDatatype(DEFAULT_TARGET_VALUE_DATATYPE)
            .interval(DEFAULT_INTERVAL)
            .targetResourceType(DEFAULT_TARGET_RESOURCE_TYPE)
            .securityFeature(DEFAULT_SECURITY_FEATURE)
            .keywords(DEFAULT_KEYWORDS);
        return securityMetric;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SecurityMetric createUpdatedEntity(EntityManager em) {
        SecurityMetric securityMetric = new SecurityMetric()
            .metricId(UPDATED_METRIC_ID)
            .category(UPDATED_CATEGORY)
            .name(UPDATED_NAME)
            .source(UPDATED_SOURCE)
            .description(UPDATED_DESCRIPTION)
            .scale(UPDATED_SCALE)
            .operator(UPDATED_OPERATOR)
            .targetValue(UPDATED_TARGET_VALUE)
            .targetValueDatatype(UPDATED_TARGET_VALUE_DATATYPE)
            .interval(UPDATED_INTERVAL)
            .targetResourceType(UPDATED_TARGET_RESOURCE_TYPE)
            .securityFeature(UPDATED_SECURITY_FEATURE)
            .keywords(UPDATED_KEYWORDS);
        return securityMetric;
    }

    @BeforeEach
    public void initTest() {
        securityMetric = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedSecurityMetric != null) {
            securityMetricRepository.delete(insertedSecurityMetric);
            insertedSecurityMetric = null;
        }
    }

    @Test
    @Transactional
    void createSecurityMetric() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the SecurityMetric
        var returnedSecurityMetric = om.readValue(
            restSecurityMetricMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            SecurityMetric.class
        );

        // Validate the SecurityMetric in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertSecurityMetricUpdatableFieldsEquals(returnedSecurityMetric, getPersistedSecurityMetric(returnedSecurityMetric));

        insertedSecurityMetric = returnedSecurityMetric;
    }

    @Test
    @Transactional
    void createSecurityMetricWithExistingId() throws Exception {
        // Create the SecurityMetric with an existing ID
        securityMetric.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkCategoryIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setCategory(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setName(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkSourceIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setSource(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkScaleIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setScale(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkOperatorIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setOperator(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkTargetValueIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setTargetValue(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkTargetValueDatatypeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setTargetValueDatatype(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkIntervalIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        securityMetric.setInterval(null);

        // Create the SecurityMetric, which fails.

        restSecurityMetricMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSecurityMetrics() throws Exception {
        // Initialize the database
        insertedSecurityMetric = securityMetricRepository.saveAndFlush(securityMetric);

        // Get all the securityMetricList
        restSecurityMetricMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(securityMetric.getId().intValue())))
            .andExpect(jsonPath("$.[*].metricId").value(hasItem(DEFAULT_METRIC_ID)))
            .andExpect(jsonPath("$.[*].category").value(hasItem(DEFAULT_CATEGORY)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].source").value(hasItem(DEFAULT_SOURCE)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION.toString())))
            .andExpect(jsonPath("$.[*].scale").value(hasItem(DEFAULT_SCALE)))
            .andExpect(jsonPath("$.[*].operator").value(hasItem(DEFAULT_OPERATOR)))
            .andExpect(jsonPath("$.[*].targetValue").value(hasItem(DEFAULT_TARGET_VALUE)))
            .andExpect(jsonPath("$.[*].targetValueDatatype").value(hasItem(DEFAULT_TARGET_VALUE_DATATYPE)))
            .andExpect(jsonPath("$.[*].interval").value(hasItem(DEFAULT_INTERVAL)))
            .andExpect(jsonPath("$.[*].targetResourceType").value(hasItem(DEFAULT_TARGET_RESOURCE_TYPE)))
            .andExpect(jsonPath("$.[*].securityFeature").value(hasItem(DEFAULT_SECURITY_FEATURE)))
            .andExpect(jsonPath("$.[*].keywords").value(hasItem(DEFAULT_KEYWORDS)));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSecurityMetricsWithEagerRelationshipsIsEnabled() throws Exception {
        when(securityMetricRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSecurityMetricMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(securityMetricRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSecurityMetricsWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(securityMetricRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSecurityMetricMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(securityMetricRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getSecurityMetric() throws Exception {
        // Initialize the database
        insertedSecurityMetric = securityMetricRepository.saveAndFlush(securityMetric);

        // Get the securityMetric
        restSecurityMetricMockMvc
            .perform(get(ENTITY_API_URL_ID, securityMetric.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(securityMetric.getId().intValue()))
            .andExpect(jsonPath("$.metricId").value(DEFAULT_METRIC_ID))
            .andExpect(jsonPath("$.category").value(DEFAULT_CATEGORY))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.source").value(DEFAULT_SOURCE))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION.toString()))
            .andExpect(jsonPath("$.scale").value(DEFAULT_SCALE))
            .andExpect(jsonPath("$.operator").value(DEFAULT_OPERATOR))
            .andExpect(jsonPath("$.targetValue").value(DEFAULT_TARGET_VALUE))
            .andExpect(jsonPath("$.targetValueDatatype").value(DEFAULT_TARGET_VALUE_DATATYPE))
            .andExpect(jsonPath("$.interval").value(DEFAULT_INTERVAL))
            .andExpect(jsonPath("$.targetResourceType").value(DEFAULT_TARGET_RESOURCE_TYPE))
            .andExpect(jsonPath("$.securityFeature").value(DEFAULT_SECURITY_FEATURE))
            .andExpect(jsonPath("$.keywords").value(DEFAULT_KEYWORDS));
    }

    @Test
    @Transactional
    void getNonExistingSecurityMetric() throws Exception {
        // Get the securityMetric
        restSecurityMetricMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingSecurityMetric() throws Exception {
        // Initialize the database
        insertedSecurityMetric = securityMetricRepository.saveAndFlush(securityMetric);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityMetric
        SecurityMetric updatedSecurityMetric = securityMetricRepository.findById(securityMetric.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedSecurityMetric are not directly saved in db
        em.detach(updatedSecurityMetric);
        updatedSecurityMetric
            .metricId(UPDATED_METRIC_ID)
            .category(UPDATED_CATEGORY)
            .name(UPDATED_NAME)
            .source(UPDATED_SOURCE)
            .description(UPDATED_DESCRIPTION)
            .scale(UPDATED_SCALE)
            .operator(UPDATED_OPERATOR)
            .targetValue(UPDATED_TARGET_VALUE)
            .targetValueDatatype(UPDATED_TARGET_VALUE_DATATYPE)
            .interval(UPDATED_INTERVAL)
            .targetResourceType(UPDATED_TARGET_RESOURCE_TYPE)
            .securityFeature(UPDATED_SECURITY_FEATURE)
            .keywords(UPDATED_KEYWORDS);

        restSecurityMetricMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedSecurityMetric.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedSecurityMetric))
            )
            .andExpect(status().isOk());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedSecurityMetricToMatchAllProperties(updatedSecurityMetric);
    }

    @Test
    @Transactional
    void putNonExistingSecurityMetric() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityMetric.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityMetricMockMvc
            .perform(
                put(ENTITY_API_URL_ID, securityMetric.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSecurityMetric() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityMetric.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityMetricMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSecurityMetric() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityMetric.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityMetricMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(securityMetric)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSecurityMetricWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityMetric = securityMetricRepository.saveAndFlush(securityMetric);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityMetric using partial update
        SecurityMetric partialUpdatedSecurityMetric = new SecurityMetric();
        partialUpdatedSecurityMetric.setId(securityMetric.getId());

        partialUpdatedSecurityMetric
            .source(UPDATED_SOURCE)
            .scale(UPDATED_SCALE)
            .operator(UPDATED_OPERATOR)
            .targetResourceType(UPDATED_TARGET_RESOURCE_TYPE)
            .securityFeature(UPDATED_SECURITY_FEATURE);

        restSecurityMetricMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityMetric.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityMetric))
            )
            .andExpect(status().isOk());

        // Validate the SecurityMetric in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityMetricUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedSecurityMetric, securityMetric),
            getPersistedSecurityMetric(securityMetric)
        );
    }

    @Test
    @Transactional
    void fullUpdateSecurityMetricWithPatch() throws Exception {
        // Initialize the database
        insertedSecurityMetric = securityMetricRepository.saveAndFlush(securityMetric);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the securityMetric using partial update
        SecurityMetric partialUpdatedSecurityMetric = new SecurityMetric();
        partialUpdatedSecurityMetric.setId(securityMetric.getId());

        partialUpdatedSecurityMetric
            .metricId(UPDATED_METRIC_ID)
            .category(UPDATED_CATEGORY)
            .name(UPDATED_NAME)
            .source(UPDATED_SOURCE)
            .description(UPDATED_DESCRIPTION)
            .scale(UPDATED_SCALE)
            .operator(UPDATED_OPERATOR)
            .targetValue(UPDATED_TARGET_VALUE)
            .targetValueDatatype(UPDATED_TARGET_VALUE_DATATYPE)
            .interval(UPDATED_INTERVAL)
            .targetResourceType(UPDATED_TARGET_RESOURCE_TYPE)
            .securityFeature(UPDATED_SECURITY_FEATURE)
            .keywords(UPDATED_KEYWORDS);

        restSecurityMetricMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSecurityMetric.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSecurityMetric))
            )
            .andExpect(status().isOk());

        // Validate the SecurityMetric in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSecurityMetricUpdatableFieldsEquals(partialUpdatedSecurityMetric, getPersistedSecurityMetric(partialUpdatedSecurityMetric));
    }

    @Test
    @Transactional
    void patchNonExistingSecurityMetric() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityMetric.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSecurityMetricMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, securityMetric.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSecurityMetric() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityMetric.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityMetricMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isBadRequest());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSecurityMetric() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        securityMetric.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSecurityMetricMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(securityMetric))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SecurityMetric in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSecurityMetric() throws Exception {
        // Initialize the database
        insertedSecurityMetric = securityMetricRepository.saveAndFlush(securityMetric);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the securityMetric
        restSecurityMetricMockMvc
            .perform(delete(ENTITY_API_URL_ID, securityMetric.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return securityMetricRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected SecurityMetric getPersistedSecurityMetric(SecurityMetric securityMetric) {
        return securityMetricRepository.findById(securityMetric.getId()).orElseThrow();
    }

    protected void assertPersistedSecurityMetricToMatchAllProperties(SecurityMetric expectedSecurityMetric) {
        assertSecurityMetricAllPropertiesEquals(expectedSecurityMetric, getPersistedSecurityMetric(expectedSecurityMetric));
    }

    protected void assertPersistedSecurityMetricToMatchUpdatableProperties(SecurityMetric expectedSecurityMetric) {
        assertSecurityMetricAllUpdatablePropertiesEquals(expectedSecurityMetric, getPersistedSecurityMetric(expectedSecurityMetric));
    }
}
