package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.SimilarControlAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.SimilarControl;
import com.emerald.rcm.backend.repository.SimilarControlRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link SimilarControlResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class SimilarControlResourceIT {

    private static final String DEFAULT_CODE = "AAAAAAAAAA";
    private static final String UPDATED_CODE = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_SECURITY_CONTROL_FRAMEWORK_NAME = "AAAAAAAAAA";
    private static final String UPDATED_SECURITY_CONTROL_FRAMEWORK_NAME = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/similar-controls";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private SimilarControlRepository similarControlRepository;

    @Mock
    private SimilarControlRepository similarControlRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSimilarControlMockMvc;

    private SimilarControl similarControl;

    private SimilarControl insertedSimilarControl;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SimilarControl createEntity(EntityManager em) {
        SimilarControl similarControl = new SimilarControl()
            .code(DEFAULT_CODE)
            .name(DEFAULT_NAME)
            .securityControlFrameworkName(DEFAULT_SECURITY_CONTROL_FRAMEWORK_NAME);
        return similarControl;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SimilarControl createUpdatedEntity(EntityManager em) {
        SimilarControl similarControl = new SimilarControl()
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .securityControlFrameworkName(UPDATED_SECURITY_CONTROL_FRAMEWORK_NAME);
        return similarControl;
    }

    @BeforeEach
    public void initTest() {
        similarControl = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedSimilarControl != null) {
            similarControlRepository.delete(insertedSimilarControl);
            insertedSimilarControl = null;
        }
    }

    @Test
    @Transactional
    void createSimilarControl() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the SimilarControl
        var returnedSimilarControl = om.readValue(
            restSimilarControlMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(similarControl))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            SimilarControl.class
        );

        // Validate the SimilarControl in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertSimilarControlUpdatableFieldsEquals(returnedSimilarControl, getPersistedSimilarControl(returnedSimilarControl));

        insertedSimilarControl = returnedSimilarControl;
    }

    @Test
    @Transactional
    void createSimilarControlWithExistingId() throws Exception {
        // Create the SimilarControl with an existing ID
        similarControl.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSimilarControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkCodeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        similarControl.setCode(null);

        // Create the SimilarControl, which fails.

        restSimilarControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        similarControl.setName(null);

        // Create the SimilarControl, which fails.

        restSimilarControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkSecurityControlFrameworkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        similarControl.setSecurityControlFrameworkName(null);

        // Create the SimilarControl, which fails.

        restSimilarControlMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSimilarControls() throws Exception {
        // Initialize the database
        insertedSimilarControl = similarControlRepository.saveAndFlush(similarControl);

        // Get all the similarControlList
        restSimilarControlMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(similarControl.getId().intValue())))
            .andExpect(jsonPath("$.[*].code").value(hasItem(DEFAULT_CODE)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].securityControlFrameworkName").value(hasItem(DEFAULT_SECURITY_CONTROL_FRAMEWORK_NAME)));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSimilarControlsWithEagerRelationshipsIsEnabled() throws Exception {
        when(similarControlRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSimilarControlMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(similarControlRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllSimilarControlsWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(similarControlRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restSimilarControlMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(similarControlRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getSimilarControl() throws Exception {
        // Initialize the database
        insertedSimilarControl = similarControlRepository.saveAndFlush(similarControl);

        // Get the similarControl
        restSimilarControlMockMvc
            .perform(get(ENTITY_API_URL_ID, similarControl.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(similarControl.getId().intValue()))
            .andExpect(jsonPath("$.code").value(DEFAULT_CODE))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.securityControlFrameworkName").value(DEFAULT_SECURITY_CONTROL_FRAMEWORK_NAME));
    }

    @Test
    @Transactional
    void getNonExistingSimilarControl() throws Exception {
        // Get the similarControl
        restSimilarControlMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingSimilarControl() throws Exception {
        // Initialize the database
        insertedSimilarControl = similarControlRepository.saveAndFlush(similarControl);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the similarControl
        SimilarControl updatedSimilarControl = similarControlRepository.findById(similarControl.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedSimilarControl are not directly saved in db
        em.detach(updatedSimilarControl);
        updatedSimilarControl.code(UPDATED_CODE).name(UPDATED_NAME).securityControlFrameworkName(UPDATED_SECURITY_CONTROL_FRAMEWORK_NAME);

        restSimilarControlMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedSimilarControl.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedSimilarControl))
            )
            .andExpect(status().isOk());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedSimilarControlToMatchAllProperties(updatedSimilarControl);
    }

    @Test
    @Transactional
    void putNonExistingSimilarControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        similarControl.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSimilarControlMockMvc
            .perform(
                put(ENTITY_API_URL_ID, similarControl.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSimilarControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        similarControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSimilarControlMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSimilarControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        similarControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSimilarControlMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(similarControl)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSimilarControlWithPatch() throws Exception {
        // Initialize the database
        insertedSimilarControl = similarControlRepository.saveAndFlush(similarControl);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the similarControl using partial update
        SimilarControl partialUpdatedSimilarControl = new SimilarControl();
        partialUpdatedSimilarControl.setId(similarControl.getId());

        partialUpdatedSimilarControl.code(UPDATED_CODE);

        restSimilarControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSimilarControl.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSimilarControl))
            )
            .andExpect(status().isOk());

        // Validate the SimilarControl in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSimilarControlUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedSimilarControl, similarControl),
            getPersistedSimilarControl(similarControl)
        );
    }

    @Test
    @Transactional
    void fullUpdateSimilarControlWithPatch() throws Exception {
        // Initialize the database
        insertedSimilarControl = similarControlRepository.saveAndFlush(similarControl);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the similarControl using partial update
        SimilarControl partialUpdatedSimilarControl = new SimilarControl();
        partialUpdatedSimilarControl.setId(similarControl.getId());

        partialUpdatedSimilarControl
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .securityControlFrameworkName(UPDATED_SECURITY_CONTROL_FRAMEWORK_NAME);

        restSimilarControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSimilarControl.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedSimilarControl))
            )
            .andExpect(status().isOk());

        // Validate the SimilarControl in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertSimilarControlUpdatableFieldsEquals(partialUpdatedSimilarControl, getPersistedSimilarControl(partialUpdatedSimilarControl));
    }

    @Test
    @Transactional
    void patchNonExistingSimilarControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        similarControl.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSimilarControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, similarControl.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSimilarControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        similarControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSimilarControlMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isBadRequest());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSimilarControl() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        similarControl.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSimilarControlMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(similarControl))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SimilarControl in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSimilarControl() throws Exception {
        // Initialize the database
        insertedSimilarControl = similarControlRepository.saveAndFlush(similarControl);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the similarControl
        restSimilarControlMockMvc
            .perform(delete(ENTITY_API_URL_ID, similarControl.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return similarControlRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected SimilarControl getPersistedSimilarControl(SimilarControl similarControl) {
        return similarControlRepository.findById(similarControl.getId()).orElseThrow();
    }

    protected void assertPersistedSimilarControlToMatchAllProperties(SimilarControl expectedSimilarControl) {
        assertSimilarControlAllPropertiesEquals(expectedSimilarControl, getPersistedSimilarControl(expectedSimilarControl));
    }

    protected void assertPersistedSimilarControlToMatchUpdatableProperties(SimilarControl expectedSimilarControl) {
        assertSimilarControlAllUpdatablePropertiesEquals(expectedSimilarControl, getPersistedSimilarControl(expectedSimilarControl));
    }
}
