package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.TomAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.Tom;
import com.emerald.rcm.backend.domain.enumeration.AssuranceLevel;
import com.emerald.rcm.backend.domain.enumeration.Type;
import com.emerald.rcm.backend.repository.TomRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link TomResource} REST controller.
 */
@IntegrationTest
@Disabled("Cyclic required relationships detected")
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class TomResourceIT {

    private static final String DEFAULT_CODE = "AAAAAAAAAA";
    private static final String UPDATED_CODE = "BBBBBBBBBB";

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_DESCRIPTION = "AAAAAAAAAA";
    private static final String UPDATED_DESCRIPTION = "BBBBBBBBBB";

    private static final AssuranceLevel DEFAULT_ASSURANCE_LEVEL = AssuranceLevel.BASIC;
    private static final AssuranceLevel UPDATED_ASSURANCE_LEVEL = AssuranceLevel.SUBSTANTIAL;

    private static final Type DEFAULT_TYPE = Type.ORGANIZATIONAL;
    private static final Type UPDATED_TYPE = Type.TECHNICAL;

    private static final String DEFAULT_GUIDANCE = "AAAAAAAAAA";
    private static final String UPDATED_GUIDANCE = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/toms";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private TomRepository tomRepository;

    @Mock
    private TomRepository tomRepositoryMock;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restTomMockMvc;

    private Tom tom;

    private Tom insertedTom;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Tom createEntity(EntityManager em) {
        Tom tom = new Tom()
            .code(DEFAULT_CODE)
            .name(DEFAULT_NAME)
            .description(DEFAULT_DESCRIPTION)
            .assuranceLevel(DEFAULT_ASSURANCE_LEVEL)
            .type(DEFAULT_TYPE)
            .guidance(DEFAULT_GUIDANCE);
        return tom;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static Tom createUpdatedEntity(EntityManager em) {
        Tom tom = new Tom()
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .description(UPDATED_DESCRIPTION)
            .assuranceLevel(UPDATED_ASSURANCE_LEVEL)
            .type(UPDATED_TYPE)
            .guidance(UPDATED_GUIDANCE);
        return tom;
    }

    @BeforeEach
    public void initTest() {
        tom = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedTom != null) {
            tomRepository.delete(insertedTom);
            insertedTom = null;
        }
    }

    @Test
    @Transactional
    void createTom() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the Tom
        var returnedTom = om.readValue(
            restTomMockMvc
                .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom)))
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            Tom.class
        );

        // Validate the Tom in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertTomUpdatableFieldsEquals(returnedTom, getPersistedTom(returnedTom));

        insertedTom = returnedTom;
    }

    @Test
    @Transactional
    void createTomWithExistingId() throws Exception {
        // Create the Tom with an existing ID
        tom.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restTomMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom)))
            .andExpect(status().isBadRequest());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkCodeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        tom.setCode(null);

        // Create the Tom, which fails.

        restTomMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        tom.setName(null);

        // Create the Tom, which fails.

        restTomMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkTypeIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        tom.setType(null);

        // Create the Tom, which fails.

        restTomMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllToms() throws Exception {
        // Initialize the database
        insertedTom = tomRepository.saveAndFlush(tom);

        // Get all the tomList
        restTomMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(tom.getId().intValue())))
            .andExpect(jsonPath("$.[*].code").value(hasItem(DEFAULT_CODE)))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].description").value(hasItem(DEFAULT_DESCRIPTION.toString())))
            .andExpect(jsonPath("$.[*].assuranceLevel").value(hasItem(DEFAULT_ASSURANCE_LEVEL.toString())))
            .andExpect(jsonPath("$.[*].type").value(hasItem(DEFAULT_TYPE.toString())))
            .andExpect(jsonPath("$.[*].guidance").value(hasItem(DEFAULT_GUIDANCE.toString())));
    }

    @SuppressWarnings({ "unchecked" })
    void getAllTomsWithEagerRelationshipsIsEnabled() throws Exception {
        when(tomRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restTomMockMvc.perform(get(ENTITY_API_URL + "?eagerload=true")).andExpect(status().isOk());

        verify(tomRepositoryMock, times(1)).findAllWithEagerRelationships(any());
    }

    @SuppressWarnings({ "unchecked" })
    void getAllTomsWithEagerRelationshipsIsNotEnabled() throws Exception {
        when(tomRepositoryMock.findAllWithEagerRelationships(any())).thenReturn(new PageImpl(new ArrayList<>()));

        restTomMockMvc.perform(get(ENTITY_API_URL + "?eagerload=false")).andExpect(status().isOk());
        verify(tomRepositoryMock, times(1)).findAll(any(Pageable.class));
    }

    @Test
    @Transactional
    void getTom() throws Exception {
        // Initialize the database
        insertedTom = tomRepository.saveAndFlush(tom);

        // Get the tom
        restTomMockMvc
            .perform(get(ENTITY_API_URL_ID, tom.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(tom.getId().intValue()))
            .andExpect(jsonPath("$.code").value(DEFAULT_CODE))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.description").value(DEFAULT_DESCRIPTION.toString()))
            .andExpect(jsonPath("$.assuranceLevel").value(DEFAULT_ASSURANCE_LEVEL.toString()))
            .andExpect(jsonPath("$.type").value(DEFAULT_TYPE.toString()))
            .andExpect(jsonPath("$.guidance").value(DEFAULT_GUIDANCE.toString()));
    }

    @Test
    @Transactional
    void getNonExistingTom() throws Exception {
        // Get the tom
        restTomMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingTom() throws Exception {
        // Initialize the database
        insertedTom = tomRepository.saveAndFlush(tom);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the tom
        Tom updatedTom = tomRepository.findById(tom.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedTom are not directly saved in db
        em.detach(updatedTom);
        updatedTom
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .description(UPDATED_DESCRIPTION)
            .assuranceLevel(UPDATED_ASSURANCE_LEVEL)
            .type(UPDATED_TYPE)
            .guidance(UPDATED_GUIDANCE);

        restTomMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedTom.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedTom))
            )
            .andExpect(status().isOk());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedTomToMatchAllProperties(updatedTom);
    }

    @Test
    @Transactional
    void putNonExistingTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        tom.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restTomMockMvc
            .perform(
                put(ENTITY_API_URL_ID, tom.getId()).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom))
            )
            .andExpect(status().isBadRequest());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        tom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTomMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(tom))
            )
            .andExpect(status().isBadRequest());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        tom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTomMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(tom)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateTomWithPatch() throws Exception {
        // Initialize the database
        insertedTom = tomRepository.saveAndFlush(tom);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the tom using partial update
        Tom partialUpdatedTom = new Tom();
        partialUpdatedTom.setId(tom.getId());

        partialUpdatedTom.type(UPDATED_TYPE).guidance(UPDATED_GUIDANCE);

        restTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedTom.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedTom))
            )
            .andExpect(status().isOk());

        // Validate the Tom in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertTomUpdatableFieldsEquals(createUpdateProxyForBean(partialUpdatedTom, tom), getPersistedTom(tom));
    }

    @Test
    @Transactional
    void fullUpdateTomWithPatch() throws Exception {
        // Initialize the database
        insertedTom = tomRepository.saveAndFlush(tom);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the tom using partial update
        Tom partialUpdatedTom = new Tom();
        partialUpdatedTom.setId(tom.getId());

        partialUpdatedTom
            .code(UPDATED_CODE)
            .name(UPDATED_NAME)
            .description(UPDATED_DESCRIPTION)
            .assuranceLevel(UPDATED_ASSURANCE_LEVEL)
            .type(UPDATED_TYPE)
            .guidance(UPDATED_GUIDANCE);

        restTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedTom.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedTom))
            )
            .andExpect(status().isOk());

        // Validate the Tom in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertTomUpdatableFieldsEquals(partialUpdatedTom, getPersistedTom(partialUpdatedTom));
    }

    @Test
    @Transactional
    void patchNonExistingTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        tom.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, tom.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(tom))
            )
            .andExpect(status().isBadRequest());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        tom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTomMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(tom))
            )
            .andExpect(status().isBadRequest());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamTom() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        tom.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTomMockMvc
            .perform(patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(tom)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the Tom in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteTom() throws Exception {
        // Initialize the database
        insertedTom = tomRepository.saveAndFlush(tom);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the tom
        restTomMockMvc
            .perform(delete(ENTITY_API_URL_ID, tom.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return tomRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected Tom getPersistedTom(Tom tom) {
        return tomRepository.findById(tom.getId()).orElseThrow();
    }

    protected void assertPersistedTomToMatchAllProperties(Tom expectedTom) {
        assertTomAllPropertiesEquals(expectedTom, getPersistedTom(expectedTom));
    }

    protected void assertPersistedTomToMatchUpdatableProperties(Tom expectedTom) {
        assertTomAllUpdatablePropertiesEquals(expectedTom, getPersistedTom(expectedTom));
    }
}
